@@ -1,65 +1,54 @@
-use std::sync::Arc;
-
 use libc::{c_void, size_t, EINVAL, EIO};
 use rustls::server::{Accepted, AcceptedAlert, Acceptor};
-use rustls::ServerConfig;
 
 use crate::connection::rustls_connection;
 use crate::error::{map_error, rustls_io_result};
-use crate::io::{
-    rustls_read_callback, rustls_write_callback, CallbackReader, CallbackWriter, ReadCallback,
-};
+use crate::io::{rustls_read_callback, rustls_write_callback, CallbackReader, CallbackWriter};
 use crate::rslice::{rustls_slice_bytes, rustls_str};
 use crate::server::rustls_server_config;
 use crate::{
-    ffi_panic_boundary, free_box, rustls_result, set_boxed_mut_ptr, to_box, to_boxed_mut_ptr,
-    try_callback, try_clone_arc, try_mut_from_ptr, try_ref_from_ptr, try_take, Castable,
-    OwnershipBox,
+    box_castable, ffi_panic_boundary, free_box, rustls_result, set_boxed_mut_ptr, to_box,
+    to_boxed_mut_ptr, try_callback, try_clone_arc, try_mut_from_ptr, try_mut_from_ptr_ptr,
+    try_ref_from_ptr, try_take,
 };
-use rustls_result::NullParameter;
 
-/// A buffer and parser for ClientHello bytes. This allows reading ClientHello
-/// before choosing a rustls_server_config. It's useful when the server
-/// config will be based on parameters in the ClientHello: server name
-/// indication (SNI), ALPN protocols, signature schemes, and cipher suites. In
-/// particular, if a server wants to do some potentially expensive work to load a
-/// certificate for a given hostname, rustls_acceptor allows doing that asynchronously,
-/// as opposed to rustls_server_config_builder_set_hello_callback(), which doesn't
-/// work well for asynchronous I/O.
-///
-/// The general flow is:
-///  - rustls_acceptor_new()
-///  - Loop:
-///    - Read bytes from the network it with rustls_acceptor_read_tls().
-///    - If successful, parse those bytes with rustls_acceptor_accept().
-///    - If that returns RUSTLS_RESULT_ACCEPTOR_NOT_READY, continue.
-///    - Otherwise, break.
-///  - If rustls_acceptor_accept() returned RUSTLS_RESULT_OK:
-///    - Examine the resulting rustls_accepted.
-///    - Create or select a rustls_server_config.
-///    - Call rustls_accepted_into_connection().
-///  - Otherwise, there was a problem with the ClientHello data and the
-///    connection should be rejected.
-pub struct rustls_acceptor {
-    _private: [u8; 0],
+box_castable! {
+    /// A buffer and parser for ClientHello bytes.
+    ///
+    /// This allows reading ClientHello before choosing a rustls_server_config.
+    ///
+    /// It's useful when the server config will be based on parameters in the
+    /// ClientHello: server name indication (SNI), ALPN protocols, signature
+    /// schemes, and cipher suites.
+    ///
+    /// In particular, if a server wants to do some potentially expensive work
+    /// to load a certificate for a given hostname, rustls_acceptor allows doing
+    /// that asynchronously, as opposed to rustls_server_config_builder_set_hello_callback(),
+    /// which doesn't work well for asynchronous I/O.
+    ///
+    /// The general flow is:
+    ///  - rustls_acceptor_new()
+    ///  - Loop:
+    ///    - Read bytes from the network it with rustls_acceptor_read_tls().
+    ///    - If successful, parse those bytes with rustls_acceptor_accept().
+    ///    - If that returns RUSTLS_RESULT_ACCEPTOR_NOT_READY, continue.
+    ///    - Otherwise, break.
+    ///  - If rustls_acceptor_accept() returned RUSTLS_RESULT_OK:
+    ///    - Examine the resulting rustls_accepted.
+    ///    - Create or select a rustls_server_config.
+    ///    - Call rustls_accepted_into_connection().
+    ///  - Otherwise, there was a problem with the ClientHello data and the
+    ///    connection should be rejected.
+    pub struct rustls_acceptor(Acceptor);
 }
 
-impl Castable for rustls_acceptor {
-    type Ownership = OwnershipBox;
-    type RustType = Acceptor;
-}
-
-/// A parsed ClientHello produced by a rustls_acceptor. It is used to check
-/// server name indication (SNI), ALPN protocols, signature schemes, and
-/// cipher suites. It can be combined with a rustls_server_config to build a
-/// rustls_connection.
-pub struct rustls_accepted {
-    _private: [u8; 0],
-}
-
-impl Castable for rustls_accepted {
-    type Ownership = OwnershipBox;
-    type RustType = Option<Accepted>;
+box_castable! {
+    /// A parsed ClientHello produced by a rustls_acceptor.
+    ///
+    /// It is used to check server name indication (SNI), ALPN protocols,
+    /// signature schemes, and cipher suites. It can be combined with a
+    /// `rustls_server_config` to build a `rustls_connection`.
+    pub struct rustls_accepted(Option<Accepted>);
 }
 
 impl rustls_acceptor {
@@ -88,10 +77,12 @@ impl rustls_acceptor {
         }
     }
 
-    /// Read some TLS bytes from the network into internal buffers. The actual network
-    /// I/O is performed by `callback`, which you provide. Rustls will invoke your
-    /// callback with a suitable buffer to store the read bytes into. You don't have
-    /// to fill it up, just fill with as many bytes as you get in one syscall.
+    /// Read some TLS bytes from the network into internal buffers.
+    ///
+    /// The actual network I/O is performed by `callback`, which you provide.
+    /// Rustls will invoke your callback with a suitable buffer to store the
+    /// read bytes into. You don't have to fill it up, just fill with as many
+    /// bytes as you get in one syscall.
     ///
     /// Parameters:
     ///
@@ -121,11 +112,11 @@ impl rustls_acceptor {
         out_n: *mut size_t,
     ) -> rustls_io_result {
         ffi_panic_boundary! {
-            let acceptor: &mut Acceptor = try_mut_from_ptr!(acceptor);
+            let acceptor = try_mut_from_ptr!(acceptor);
             if out_n.is_null() {
                 return rustls_io_result(EINVAL);
             }
-            let callback: ReadCallback = try_callback!(callback);
+            let callback = try_callback!(callback);
 
             let mut reader = CallbackReader { callback, userdata };
 
@@ -141,8 +132,9 @@ impl rustls_acceptor {
         }
     }
 
-    /// Parse all TLS bytes read so far.  If those bytes make up a ClientHello,
-    /// create a rustls_accepted from them.
+    /// Parse all TLS bytes read so far.
+    ///
+    /// If those bytes make up a ClientHello, create a rustls_accepted from them.
     ///
     /// Parameters:
     ///
@@ -186,13 +178,9 @@ impl rustls_acceptor {
         out_alert: *mut *mut rustls_accepted_alert,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let acceptor: &mut Acceptor = try_mut_from_ptr!(acceptor);
-            if out_accepted.is_null() {
-                return NullParameter;
-            }
-            if out_alert.is_null() {
-                return NullParameter;
-            }
+            let acceptor = try_mut_from_ptr!(acceptor);
+            let out_accepted = try_mut_from_ptr_ptr!(out_accepted);
+            let out_alert = try_mut_from_ptr_ptr!(out_alert);
             match acceptor.accept() {
                 Ok(None) => rustls_result::AcceptorNotReady,
                 Ok(Some(accepted)) => {
@@ -202,7 +190,7 @@ impl rustls_acceptor {
                 Err((e, accepted_alert)) => {
                     set_boxed_mut_ptr(out_alert, accepted_alert);
                     map_error(e)
-                },
+                }
             }
         }
     }
@@ -234,7 +222,7 @@ impl rustls_accepted {
         accepted: *const rustls_accepted,
     ) -> rustls_str<'static> {
         ffi_panic_boundary! {
-            let accepted: &Option<Accepted> = try_ref_from_ptr!(accepted);
+            let accepted = try_ref_from_ptr!(accepted);
             let accepted = match accepted {
                 Some(a) => a,
                 None => return Default::default(),
@@ -252,6 +240,7 @@ impl rustls_accepted {
     }
 
     /// Get the i'th in the list of signature schemes offered in the ClientHello.
+    ///
     /// This is useful in selecting a server certificate when there are multiple
     /// available for the same server name, for instance when selecting
     /// between an RSA and an ECDSA certificate.
@@ -275,7 +264,7 @@ impl rustls_accepted {
         i: usize,
     ) -> u16 {
         ffi_panic_boundary! {
-            let accepted: &Option<Accepted> = try_ref_from_ptr!(accepted);
+            let accepted = try_ref_from_ptr!(accepted);
             let accepted = match accepted {
                 Some(a) => a,
                 None => return 0,
@@ -313,7 +302,7 @@ impl rustls_accepted {
         i: usize,
     ) -> u16 {
         ffi_panic_boundary! {
-            let accepted: &Option<Accepted> = try_ref_from_ptr!(accepted);
+            let accepted = try_ref_from_ptr!(accepted);
             let accepted = match accepted {
                 Some(a) => a,
                 None => return 0,
@@ -356,7 +345,7 @@ impl rustls_accepted {
         i: usize,
     ) -> rustls_slice_bytes<'static> {
         ffi_panic_boundary! {
-            let accepted: &Option<Accepted> = try_ref_from_ptr!(accepted);
+            let accepted = try_ref_from_ptr!(accepted);
             let accepted = match accepted {
                 Some(a) => a,
                 None => return Default::default(),
@@ -423,15 +412,11 @@ impl rustls_accepted {
         out_alert: *mut *mut rustls_accepted_alert,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let accepted: &mut Option<Accepted> = try_mut_from_ptr!(accepted);
+            let accepted = try_mut_from_ptr!(accepted);
             let accepted = try_take!(accepted);
-            let config: Arc<ServerConfig> = try_clone_arc!(config);
-            if out_conn.is_null() {
-                return NullParameter;
-            }
-            if out_alert.is_null() {
-                return NullParameter;
-            }
+            let config = try_clone_arc!(config);
+            let out_conn = try_mut_from_ptr_ptr!(out_conn);
+            let out_alert = try_mut_from_ptr_ptr!(out_alert);
             match accepted.into_connection(config) {
                 Ok(built) => {
                     let wrapped = crate::connection::Connection::from_server(built);
@@ -441,7 +426,7 @@ impl rustls_accepted {
                 Err((e, accepted_alert)) => {
                     set_boxed_mut_ptr(out_alert, accepted_alert);
                     map_error(e)
-                },
+                }
             }
         }
     }
@@ -461,24 +446,23 @@ impl rustls_accepted {
     }
 }
 
-/// Represents a TLS alert resulting from accepting a client.
-pub struct rustls_accepted_alert {
-    _private: [u8; 0],
-}
-
-impl Castable for rustls_accepted_alert {
-    type Ownership = OwnershipBox;
-    type RustType = AcceptedAlert;
+box_castable! {
+    /// Represents a TLS alert resulting from accepting a client.
+    pub struct rustls_accepted_alert(AcceptedAlert);
 }
 
 impl rustls_accepted_alert {
-    /// Write some TLS bytes (an alert) to the network. The actual network I/O is
-    /// performed by `callback`, which you provide. Rustls will invoke your callback with a
-    /// suitable buffer containing TLS bytes to send. You don't have to write them
-    /// all, just as many as you can in one syscall.
+    /// Write some TLS bytes (an alert) to the network.
+    ///
+    /// The actual network I/O is performed by `callback`, which you provide.
+    /// Rustls will invoke your callback with a suitable buffer containing TLS
+    /// bytes to send. You don't have to write them all, just as many as you can
+    /// in one syscall.
+    ///
     /// The `userdata` parameter is passed through directly to `callback`. Note that
     /// this is distinct from the `userdata` parameter set with
     /// `rustls_connection_set_userdata`.
+    ///
     /// Returns 0 for success, or an errno value on error. Passes through return values
     /// from callback. See [`rustls_write_callback`] or [`AcceptedAlert`] for
     /// more details.
@@ -493,9 +477,12 @@ impl rustls_accepted_alert {
             if out_n.is_null() {
                 return rustls_io_result(EINVAL);
             }
-            let accepted_alert: &mut AcceptedAlert = try_mut_from_ptr!(accepted_alert);
-            let mut writer = CallbackWriter { callback: try_callback!(callback), userdata };
-            let n_written: usize = match accepted_alert.write(&mut writer) {
+            let accepted_alert = try_mut_from_ptr!(accepted_alert);
+            let mut writer = CallbackWriter {
+                callback: try_callback!(callback),
+                userdata,
+            };
+            let n_written = match accepted_alert.write(&mut writer) {
                 Ok(n) => n,
                 Err(e) => return rustls_io_result(e.raw_os_error().unwrap_or(EIO)),
             };
@@ -531,9 +518,9 @@ mod tests {
     use libc::c_char;
     use rustls::internal::msgs::codec::Codec;
     use rustls::internal::msgs::enums::AlertLevel;
-    use rustls::{AlertDescription, ContentType, ProtocolVersion};
+    use rustls::{AlertDescription, ContentType, ProtocolVersion, SignatureScheme};
 
-    use crate::cipher::rustls_certified_key;
+    use crate::cipher::{rustls_certified_key, rustls_server_cert_verifier};
     use crate::client::{rustls_client_config, rustls_client_config_builder};
     use crate::server::rustls_server_config_builder;
 
@@ -541,7 +528,7 @@ mod tests {
 
     #[test]
     fn test_acceptor_new_and_free() {
-        let acceptor: *mut rustls_acceptor = rustls_acceptor::rustls_acceptor_new();
+        let acceptor = rustls_acceptor::rustls_acceptor_new();
         rustls_acceptor::rustls_acceptor_free(acceptor);
     }
 
@@ -557,7 +544,7 @@ mod tests {
     ) -> rustls_io_result {
         let vecdeq: *mut VecDeque<u8> = userdata as *mut _;
         (*vecdeq).make_contiguous();
-        let first: &[u8] = (*vecdeq).as_slices().0;
+        let first = (*vecdeq).as_slices().0;
         let n = min(n, first.len());
         std::ptr::copy_nonoverlapping(first.as_ptr(), buf, n);
         (*vecdeq).drain(0..n).count();
@@ -584,9 +571,9 @@ mod tests {
     fn test_acceptor_corrupt_message() {
         let acceptor = make_acceptor();
 
-        let mut accepted: *mut rustls_accepted = null_mut();
-        let mut accepted_alert: *mut rustls_accepted_alert = null_mut();
-        let mut n: usize = 0;
+        let mut accepted = null_mut();
+        let mut accepted_alert = null_mut();
+        let mut n = 0_usize;
         let mut data = VecDeque::new();
         for _ in 0..1024 {
             data.push_back(0u8);
@@ -644,7 +631,7 @@ mod tests {
         type conn = rustls_connection;
         let builder = ccb::rustls_client_config_builder_new();
         let protocols: Vec<Vec<u8>> = vec!["zarp".into(), "yuun".into()];
-        let mut protocols_slices: Vec<rustls_slice_bytes> = vec![];
+        let mut protocols_slices = vec![];
         for p in &protocols {
             protocols_slices.push(p.as_slice().into());
         }
@@ -654,8 +641,20 @@ mod tests {
             protocols_slices.len(),
         );
 
-        let config = ccb::rustls_client_config_builder_build(builder);
-        let mut client_conn: *mut conn = null_mut();
+        let mut verifier = null_mut();
+        let result =
+            rustls_server_cert_verifier::rustls_platform_server_cert_verifier(&mut verifier);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!verifier.is_null());
+        rustls_client_config_builder::rustls_client_config_builder_set_server_verifier(
+            builder, verifier,
+        );
+
+        let mut config = null();
+        let result = ccb::rustls_client_config_builder_build(builder, &mut config);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!config.is_null());
+        let mut client_conn = null_mut();
         let result = rustls_client_config::rustls_client_connection_new(
             config,
             "example.com\0".as_ptr() as *const c_char,
@@ -665,7 +664,7 @@ mod tests {
         assert_ne!(client_conn, null_mut());
 
         let mut buf = VecDeque::<u8>::new();
-        let mut n: usize = 0;
+        let mut n = 0_usize;
         conn::rustls_connection_write_tls(
             client_conn,
             Some(vecdeque_write),
@@ -675,15 +674,15 @@ mod tests {
 
         rustls_connection::rustls_connection_free(client_conn);
         rustls_client_config::rustls_client_config_free(config);
+        rustls_server_cert_verifier::rustls_server_cert_verifier_free(verifier);
         buf
     }
 
     fn make_server_config() -> *const rustls_server_config {
-        let builder: *mut rustls_server_config_builder =
-            rustls_server_config_builder::rustls_server_config_builder_new();
+        let builder = rustls_server_config_builder::rustls_server_config_builder_new();
         let cert_pem = include_str!("../testdata/example.com/cert.pem").as_bytes();
         let key_pem = include_str!("../testdata/example.com/key.pem").as_bytes();
-        let mut certified_key: *const rustls_certified_key = null();
+        let mut certified_key = null();
         let result = rustls_certified_key::rustls_certified_key_build(
             cert_pem.as_ptr(),
             cert_pem.len(),
@@ -700,8 +699,11 @@ mod tests {
         assert_eq!(result, rustls_result::Ok);
         rustls_certified_key::rustls_certified_key_free(certified_key);
 
-        let config = rustls_server_config_builder::rustls_server_config_builder_build(builder);
-        assert_ne!(config, null());
+        let mut config = null();
+        let res =
+            rustls_server_config_builder::rustls_server_config_builder_build(builder, &mut config);
+        assert_eq!(res, rustls_result::Ok);
+        assert!(!config.is_null());
         config
     }
 
@@ -711,9 +713,9 @@ mod tests {
     fn test_acceptor_success() {
         let acceptor = make_acceptor();
 
-        let mut accepted: *mut rustls_accepted = null_mut();
-        let mut accepted_alert: *mut rustls_accepted_alert = null_mut();
-        let mut n: usize = 0;
+        let mut accepted = null_mut();
+        let mut accepted_alert = null_mut();
+        let mut n = 0;
         let mut data = client_hello_bytes();
         let data_len = data.len();
 
@@ -734,11 +736,11 @@ mod tests {
         assert_eq!(accepted_alert, null_mut());
 
         let sni = rustls_accepted::rustls_accepted_server_name(accepted);
-        let sni_as_slice = unsafe { std::slice::from_raw_parts(sni.data as *const u8, sni.len) };
+        let sni_as_slice = unsafe { slice::from_raw_parts(sni.data as *const u8, sni.len) };
         let sni_as_str = std::str::from_utf8(sni_as_slice).unwrap_or("%!(ERROR)");
         assert_eq!(sni_as_str, "example.com");
 
-        let mut signature_schemes: Vec<u16> = vec![];
+        let mut signature_schemes = vec![];
         for i in 0.. {
             let s = rustls_accepted::rustls_accepted_signature_scheme(accepted, i);
             if s == 0 {
@@ -748,12 +750,30 @@ mod tests {
         }
         // Sort to ensure consistent comparison
         signature_schemes.sort();
-        assert_eq!(
-            &signature_schemes,
-            &[1025, 1027, 1281, 1283, 1537, 2052, 2053, 2054, 2055]
-        );
 
-        let mut alpn: Vec<rustls_slice_bytes> = vec![];
+        #[cfg_attr(feature = "ring", allow(unused_mut))]
+        let mut expected_schemes = vec![
+            SignatureScheme::RSA_PKCS1_SHA256,
+            SignatureScheme::ECDSA_NISTP256_SHA256,
+            SignatureScheme::RSA_PKCS1_SHA384,
+            SignatureScheme::ECDSA_NISTP384_SHA384,
+            SignatureScheme::RSA_PKCS1_SHA512,
+            SignatureScheme::RSA_PSS_SHA256,
+            SignatureScheme::RSA_PSS_SHA384,
+            SignatureScheme::RSA_PSS_SHA512,
+            SignatureScheme::ED25519,
+        ];
+        #[cfg(feature = "aws-lc-rs")] // aws-lc-rs also includes P-521.
+        expected_schemes.push(SignatureScheme::ECDSA_NISTP521_SHA512);
+
+        let mut expected_schemes = expected_schemes
+            .into_iter()
+            .map(u16::from)
+            .collect::<Vec<_>>();
+        expected_schemes.sort();
+        assert_eq!(signature_schemes, expected_schemes);
+
+        let mut alpn = vec![];
         for i in 0.. {
             let a = rustls_accepted::rustls_accepted_alpn(accepted, i);
             if a.len == 0 {
@@ -764,13 +784,13 @@ mod tests {
 
         assert_eq!(alpn.len(), 2);
         // No need to sort ALPN because order is determine by what the client sent.
-        let alpn0 = unsafe { std::slice::from_raw_parts(alpn[0].data, alpn[0].len) };
-        let alpn1 = unsafe { std::slice::from_raw_parts(alpn[1].data, alpn[1].len) };
+        let alpn0 = unsafe { slice::from_raw_parts(alpn[0].data, alpn[0].len) };
+        let alpn1 = unsafe { slice::from_raw_parts(alpn[1].data, alpn[1].len) };
         assert_eq!(alpn0, "zarp".as_bytes());
         assert_eq!(alpn1, "yuun".as_bytes());
 
         let server_config = make_server_config();
-        let mut conn: *mut rustls_connection = null_mut();
+        let mut conn = null_mut();
         let result = rustls_accepted::rustls_accepted_into_connection(
             accepted,
             server_config,
