@@ -7,125 +7,80 @@ use libc::{c_char, size_t};
 use pki_types::{CertificateDer, UnixTime};
 use rustls::client::danger::{HandshakeSignatureValid, ServerCertVerified, ServerCertVerifier};
 use rustls::client::ResolvesClientCert;
-use rustls::crypto::ring::ALL_CIPHER_SUITES;
+use rustls::crypto::{verify_tls12_signature, verify_tls13_signature, CryptoProvider};
 use rustls::{
-    sign::CertifiedKey, CertificateError, ClientConfig, ClientConnection, DigitallySignedStruct,
-    Error, ProtocolVersion, SignatureScheme, SupportedCipherSuite, WantsVerifier,
+    sign::CertifiedKey, ClientConfig, ClientConnection, DigitallySignedStruct, Error,
+    ProtocolVersion, SignatureScheme, SupportedProtocolVersion,
 };
 
-use crate::cipher::{
-    rustls_certified_key, rustls_server_cert_verifier, rustls_supported_ciphersuite,
-};
+use crate::cipher::{rustls_certified_key, rustls_server_cert_verifier};
 use crate::connection::{rustls_connection, Connection};
+use crate::crypto_provider::rustls_crypto_provider;
 use crate::error::rustls_result::{InvalidParameter, NullParameter};
-use crate::error::{self, rustls_result};
+use crate::error::{self, map_error, rustls_result};
 use crate::rslice::NulByte;
 use crate::rslice::{rustls_slice_bytes, rustls_slice_slice_bytes, rustls_str};
 use crate::{
-    ffi_panic_boundary, free_arc, free_box, set_boxed_mut_ptr, to_arc_const_ptr, to_boxed_mut_ptr,
-    try_box_from_ptr, try_clone_arc, try_mut_from_ptr, try_ref_from_ptr, try_slice, userdata_get,
-    Castable, OwnershipArc, OwnershipBox,
+    arc_castable, box_castable, crypto_provider, ffi_panic_boundary, free_arc, free_box,
+    set_arc_mut_ptr, set_boxed_mut_ptr, to_boxed_mut_ptr, try_box_from_ptr, try_clone_arc,
+    try_mut_from_ptr, try_mut_from_ptr_ptr, try_ref_from_ptr, try_ref_from_ptr_ptr, try_slice,
+    userdata_get,
 };
 
-/// A client config being constructed. A builder can be modified by,
-/// e.g. rustls_client_config_builder_load_roots_from_file. Once you're
-/// done configuring settings, call rustls_client_config_builder_build
-/// to turn it into a *rustls_client_config. This object is not safe
-/// for concurrent mutation. Under the hood, it corresponds to a
-/// `Box<ClientConfig>`.
-/// <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
-pub struct rustls_client_config_builder {
-    _private: [u8; 0],
+box_castable! {
+    /// A client config being constructed.
+    ///
+    /// A builder can be modified by, e.g. `rustls_client_config_builder_load_roots_from_file`.
+    /// Once you're done configuring settings, call `rustls_client_config_builder_build`
+    /// to turn it into a *rustls_client_config.
+    ///
+    /// Alternatively, if an error occurs or, you don't wish to build a config,
+    /// call `rustls_client_config_builder_free` to free the builder directly.
+    ///
+    /// This object is not safe for concurrent mutation. Under the hood,
+    /// it corresponds to a `Box<ClientConfig>`.
+    /// <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
+    pub struct rustls_client_config_builder(ClientConfigBuilder);
 }
 
 pub(crate) struct ClientConfigBuilder {
-    base: rustls::ConfigBuilder<ClientConfig, WantsVerifier>,
-    verifier: Arc<dyn ServerCertVerifier>,
+    provider: Option<Arc<CryptoProvider>>,
+    versions: Vec<&'static SupportedProtocolVersion>,
+    verifier: Option<Arc<dyn ServerCertVerifier>>,
     alpn_protocols: Vec<Vec<u8>>,
     enable_sni: bool,
-    cert_resolver: Option<Arc<dyn rustls::client::ResolvesClientCert>>,
+    cert_resolver: Option<Arc<dyn ResolvesClientCert>>,
 }
 
-impl Castable for rustls_client_config_builder {
-    type Ownership = OwnershipBox;
-    type RustType = ClientConfigBuilder;
-}
-
-/// A client config that is done being constructed and is now read-only.
-/// Under the hood, this object corresponds to an `Arc<ClientConfig>`.
-/// <https://docs.rs/rustls/latest/rustls/struct.ClientConfig.html>
-pub struct rustls_client_config {
-    _private: [u8; 0],
-}
-
-impl Castable for rustls_client_config {
-    type Ownership = OwnershipArc;
-    type RustType = ClientConfig;
-}
-
-#[derive(Debug)]
-struct NoneVerifier;
-
-impl ServerCertVerifier for NoneVerifier {
-    fn verify_server_cert(
-        &self,
-        _end_entity: &CertificateDer,
-        _intermediates: &[CertificateDer],
-        _server_name: &pki_types::ServerName<'_>,
-        _ocsp_response: &[u8],
-        _now: UnixTime,
-    ) -> Result<ServerCertVerified, rustls::Error> {
-        Err(rustls::Error::InvalidCertificate(
-            CertificateError::BadSignature,
-        ))
-    }
-
-    fn verify_tls12_signature(
-        &self,
-        _message: &[u8],
-        _cert: &CertificateDer,
-        _dss: &DigitallySignedStruct,
-    ) -> Result<HandshakeSignatureValid, Error> {
-        Err(Error::InvalidCertificate(CertificateError::BadSignature))
-    }
-
-    fn verify_tls13_signature(
-        &self,
-        _message: &[u8],
-        _cert: &CertificateDer,
-        _dss: &DigitallySignedStruct,
-    ) -> Result<HandshakeSignatureValid, Error> {
-        Err(Error::InvalidCertificate(CertificateError::BadSignature))
-    }
-
-    fn supported_verify_schemes(&self) -> Vec<SignatureScheme> {
-        rustls::crypto::ring::default_provider()
-            .signature_verification_algorithms
-            .supported_schemes()
-    }
+arc_castable! {
+    /// A client config that is done being constructed and is now read-only.
+    ///
+    /// Under the hood, this object corresponds to an `Arc<ClientConfig>`.
+    /// <https://docs.rs/rustls/latest/rustls/struct.ClientConfig.html>
+    pub struct rustls_client_config(ClientConfig);
 }
 
 impl rustls_client_config_builder {
-    /// Create a rustls_client_config_builder. Caller owns the memory and must
-    /// eventually call rustls_client_config_builder_build, then free the
-    /// resulting rustls_client_config.
-    /// This uses rustls safe default values
-    /// for the cipher suites, key exchange groups and protocol versions.
-    /// This starts out with no trusted roots.
-    /// Caller must add roots with rustls_client_config_builder_load_roots_from_file
-    /// or provide a custom verifier.
+    /// Create a rustls_client_config_builder using the process default crypto provider.
+    ///
+    /// Caller owns the memory and must eventually call `rustls_client_config_builder_build`,
+    /// then free the resulting `rustls_client_config`.
+    ///
+    /// Alternatively, if an error occurs or, you don't wish to build a config,
+    /// call `rustls_client_config_builder_free` to free the builder directly.
+    ///
+    /// This uses the process default provider's values for the cipher suites and key
+    /// exchange groups, as well as safe defaults for protocol versions.
+    ///
+    /// This starts out with no trusted roots. Caller must add roots with
+    /// rustls_client_config_builder_load_roots_from_file or provide a custom verifier.
     #[no_mangle]
     pub extern "C" fn rustls_client_config_builder_new() -> *mut rustls_client_config_builder {
         ffi_panic_boundary! {
-            // Unwrap safety: *ring* default provider always has ciphersuites compatible with the
-            // default protocol versions.
-            let base =
-                ClientConfig::builder_with_provider(rustls::crypto::ring::default_provider().into())
-                    .with_safe_default_protocol_versions()
-                    .unwrap();
             let builder = ClientConfigBuilder {
-                base,
-                verifier: Arc::new(NoneVerifier),
+                provider: crypto_provider::get_default_or_install_from_crate_features(),
+                versions: rustls::DEFAULT_VERSIONS.to_vec(),
+                verifier: None,
                 cert_resolver: None,
                 alpn_protocols: vec![],
                 enable_sni: true,
@@ -134,46 +89,36 @@ impl rustls_client_config_builder {
         }
     }
 
-    /// Create a rustls_client_config_builder. Caller owns the memory and must
-    /// eventually call rustls_client_config_builder_build, then free the
-    /// resulting rustls_client_config. Specify cipher suites in preference
-    /// order; the `cipher_suites` parameter must point to an array containing
-    /// `len` pointers to `rustls_supported_ciphersuite` previously obtained
-    /// from `rustls_all_ciphersuites_get_entry()`, or to a provided array,
-    /// RUSTLS_DEFAULT_CIPHER_SUITES or RUSTLS_ALL_CIPHER_SUITES. Set the TLS
-    /// protocol versions to use when negotiating a TLS session.
+    /// Create a rustls_client_config_builder using the specified crypto provider.
     ///
+    /// Caller owns the memory and must eventually call `rustls_client_config_builder_build`,
+    /// then free the resulting `rustls_client_config`.
+    ///
+    /// Alternatively, if an error occurs or, you don't wish to build a config,
+    /// call `rustls_client_config_builder_free` to free the builder directly.
+    ///
+    /// `tls_version` sets the TLS protocol versions to use when negotiating a TLS session.
     /// `tls_version` is the version of the protocol, as defined in rfc8446,
     /// ch. 4.2.1 and end of ch. 5.1. Some values are defined in
     /// `rustls_tls_version` for convenience, and the arrays
     /// RUSTLS_DEFAULT_VERSIONS or RUSTLS_ALL_VERSIONS can be used directly.
     ///
-    /// `versions` will only be used during the call and the application retains
-    /// ownership. `len` is the number of consecutive `uint16_t` pointed to by `versions`.
+    /// `tls_versions` will only be used during the call and the application retains
+    /// ownership. `tls_versions_len` is the number of consecutive `uint16_t`
+    /// pointed to by `tls_versions`.
+    ///
+    /// Ciphersuites are configured separately via the crypto provider. See
+    /// `rustls_crypto_provider_builder_set_cipher_suites` for more information.
     #[no_mangle]
     pub extern "C" fn rustls_client_config_builder_new_custom(
-        cipher_suites: *const *const rustls_supported_ciphersuite,
-        cipher_suites_len: size_t,
+        provider: *const rustls_crypto_provider,
         tls_versions: *const u16,
         tls_versions_len: size_t,
         builder_out: *mut *mut rustls_client_config_builder,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            if builder_out.is_null() {
-                return NullParameter;
-            }
-            let cipher_suites: &[*const rustls_supported_ciphersuite] =
-                try_slice!(cipher_suites, cipher_suites_len);
-            let mut cs_vec: Vec<SupportedCipherSuite> = Vec::new();
-            for &cs in cipher_suites.iter() {
-                let cs = try_ref_from_ptr!(cs);
-                match ALL_CIPHER_SUITES.iter().find(|&acs| cs.eq(acs)) {
-                    Some(scs) => cs_vec.push(*scs),
-                    None => return InvalidParameter,
-                }
-            }
-
-            let tls_versions: &[u16] = try_slice!(tls_versions, tls_versions_len);
+            let provider = try_clone_arc!(provider);
+            let tls_versions = try_slice!(tls_versions, tls_versions_len);
             let mut versions = vec![];
             for version_number in tls_versions {
                 let proto = ProtocolVersion::from(*version_number);
@@ -183,20 +128,12 @@ impl rustls_client_config_builder {
                     versions.push(&rustls::version::TLS13);
                 }
             }
+            let builder_out = try_mut_from_ptr_ptr!(builder_out);
 
-            let provider = rustls::crypto::CryptoProvider {
-                cipher_suites: cs_vec,
-                ..rustls::crypto::ring::default_provider()
-            };
-            let result = rustls::ClientConfig::builder_with_provider(provider.into())
-                .with_protocol_versions(&versions);
-            let base = match result {
-                Ok(new) => new,
-                Err(_) => return rustls_result::InvalidParameter,
-            };
             let config_builder = ClientConfigBuilder {
-                base,
-                verifier: Arc::new(NoneVerifier),
+                provider: Some(provider),
+                versions,
+                verifier: None,
                 cert_resolver: None,
                 alpn_protocols: vec![],
                 enable_sni: true,
@@ -208,8 +145,9 @@ impl rustls_client_config_builder {
     }
 }
 
-/// Input to a custom certificate verifier callback. See
-/// rustls_client_config_builder_dangerous_set_certificate_verifier().
+/// Input to a custom certificate verifier callback.
+///
+/// See `rustls_client_config_builder_dangerous_set_certificate_verifier()`.
 ///
 /// server_name can contain a hostname, an IPv4 address in textual form, or an
 /// IPv6 address in textual form.
@@ -222,8 +160,9 @@ pub struct rustls_verify_server_cert_params<'a> {
     pub ocsp_response: rustls_slice_bytes<'a>,
 }
 
-/// User-provided input to a custom certificate verifier callback. See
-/// rustls_client_config_builder_dangerous_set_certificate_verifier().
+/// User-provided input to a custom certificate verifier callback.
+///
+/// See `rustls_client_config_builder_dangerous_set_certificate_verifier()`.
 #[allow(non_camel_case_types)]
 pub type rustls_verify_server_cert_user_data = *mut libc::c_void;
 
@@ -248,12 +187,14 @@ type VerifyCallback = unsafe extern "C" fn(
 
 // An implementation of rustls::ServerCertVerifier based on a C callback.
 struct Verifier {
+    provider: Arc<CryptoProvider>,
     callback: VerifyCallback,
 }
 
 /// Safety: Verifier is Send because we don't allocate or deallocate any of its
 /// fields.
 unsafe impl Send for Verifier {}
+
 /// Safety: Verifier is Sync if the C code that passes us a callback that
 /// obeys the concurrency safety requirements documented in
 /// rustls_client_config_builder_dangerous_set_certificate_verifier.
@@ -267,12 +208,12 @@ impl ServerCertVerifier for Verifier {
         server_name: &pki_types::ServerName<'_>,
         ocsp_response: &[u8],
         _now: UnixTime,
-    ) -> Result<ServerCertVerified, rustls::Error> {
+    ) -> Result<ServerCertVerified, Error> {
         let cb = self.callback;
         let server_name = server_name.to_str();
-        let server_name: rustls_str = match server_name.as_ref().try_into() {
+        let server_name = match server_name.as_ref().try_into() {
             Ok(r) => r,
-            Err(NulByte {}) => return Err(rustls::Error::General("NUL byte in SNI".to_string())),
+            Err(NulByte {}) => return Err(Error::General("NUL byte in SNI".to_string())),
         };
 
         let intermediates: Vec<_> = intermediates.iter().map(|cert| cert.as_ref()).collect();
@@ -287,10 +228,9 @@ impl ServerCertVerifier for Verifier {
             server_name,
             ocsp_response: ocsp_response.into(),
         };
-        let userdata = userdata_get().map_err(|_| {
-            rustls::Error::General("internal error with thread-local storage".to_string())
-        })?;
-        let result: u32 = unsafe { cb(userdata, &params) };
+        let userdata = userdata_get()
+            .map_err(|_| Error::General("internal error with thread-local storage".to_string()))?;
+        let result = unsafe { cb(userdata, &params) };
         match rustls_result::from(result) {
             rustls_result::Ok => Ok(ServerCertVerified::assertion()),
             r => Err(error::cert_result_to_error(r)),
@@ -303,11 +243,11 @@ impl ServerCertVerifier for Verifier {
         cert: &CertificateDer,
         dss: &DigitallySignedStruct,
     ) -> Result<HandshakeSignatureValid, Error> {
-        rustls::crypto::verify_tls12_signature(
+        verify_tls12_signature(
             message,
             cert,
             dss,
-            &rustls::crypto::ring::default_provider().signature_verification_algorithms,
+            &self.provider.signature_verification_algorithms,
         )
     }
 
@@ -317,16 +257,16 @@ impl ServerCertVerifier for Verifier {
         cert: &CertificateDer,
         dss: &DigitallySignedStruct,
     ) -> Result<HandshakeSignatureValid, Error> {
-        rustls::crypto::verify_tls13_signature(
+        verify_tls13_signature(
             message,
             cert,
             dss,
-            &rustls::crypto::ring::default_provider().signature_verification_algorithms,
+            &self.provider.signature_verification_algorithms,
         )
     }
 
     fn supported_verify_schemes(&self) -> Vec<SignatureScheme> {
-        rustls::crypto::ring::default_provider()
+        self.provider
             .signature_verification_algorithms
             .supported_schemes()
     }
@@ -339,7 +279,10 @@ impl Debug for Verifier {
 }
 
 impl rustls_client_config_builder {
-    /// Set a custom server certificate verifier.
+    /// Set a custom server certificate verifier using the builder crypto provider.
+    /// Returns rustls_result::NoDefaultCryptoProvider if no process default crypto
+    /// provider has been set, and the builder was not constructed with an explicit
+    /// provider choice.
     ///
     /// The callback must not capture any of the pointers in its
     /// rustls_verify_server_cert_params.
@@ -370,13 +313,17 @@ impl rustls_client_config_builder {
     ) -> rustls_result {
         ffi_panic_boundary! {
             let config_builder = try_mut_from_ptr!(config_builder);
-            let callback: VerifyCallback = match callback {
+            let callback = match callback {
                 Some(cb) => cb,
-                None => return rustls_result::InvalidParameter,
+                None => return InvalidParameter,
             };
 
-            let verifier: Verifier = Verifier { callback };
-            config_builder.verifier = Arc::new(verifier);
+            let provider = match &config_builder.provider {
+                Some(provider) => provider.clone(),
+                None => return rustls_result::NoDefaultCryptoProvider,
+            };
+
+            config_builder.verifier = Some(Arc::new(Verifier { provider, callback }));
             rustls_result::Ok
         }
     }
@@ -390,17 +337,20 @@ impl rustls_client_config_builder {
         verifier: *const rustls_server_cert_verifier,
     ) {
         ffi_panic_boundary! {
-            let builder: &mut ClientConfigBuilder = try_mut_from_ptr!(builder);
-            let verifier = try_ref_from_ptr!(verifier);
-            builder.verifier = verifier.clone();
+            let builder = try_mut_from_ptr!(builder);
+            builder.verifier = Some(try_ref_from_ptr!(verifier).clone());
         }
     }
 
-    /// Set the ALPN protocol list to the given protocols. `protocols` must point
-    /// to a buffer of `rustls_slice_bytes` (built by the caller) with `len`
-    /// elements. Each element of the buffer must be a rustls_slice_bytes whose
-    /// data field points to a single ALPN protocol ID. Standard ALPN protocol
-    /// IDs are defined at
+    /// Set the ALPN protocol list to the given protocols.
+    ///
+    /// `protocols` must point to a buffer of `rustls_slice_bytes` (built by the caller) with `len`
+    /// elements.
+    ///
+    /// Each element of the buffer must be a rustls_slice_bytes whose
+    /// data field points to a single ALPN protocol ID.
+    ///
+    /// Standard ALPN protocol IDs are defined at
     /// <https://www.iana.org/assignments/tls-extensiontype-values/tls-extensiontype-values.xhtml#alpn-protocol-ids>.
     ///
     /// This function makes a copy of the data in `protocols` and does not retain
@@ -414,12 +364,12 @@ impl rustls_client_config_builder {
         len: size_t,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let config: &mut ClientConfigBuilder = try_mut_from_ptr!(builder);
-            let protocols: &[rustls_slice_bytes] = try_slice!(protocols, len);
+            let config = try_mut_from_ptr!(builder);
+            let protocols = try_slice!(protocols, len);
 
-            let mut vv: Vec<Vec<u8>> = Vec::with_capacity(protocols.len());
+            let mut vv = Vec::with_capacity(protocols.len());
             for p in protocols {
-                let v: &[u8] = try_slice!(p.data, p.len);
+                let v = try_slice!(p.data, p.len);
                 vv.push(v.to_vec());
             }
             config.alpn_protocols = vv;
@@ -435,15 +385,17 @@ impl rustls_client_config_builder {
         enable: bool,
     ) {
         ffi_panic_boundary! {
-            let config: &mut ClientConfigBuilder = try_mut_from_ptr!(config);
+            let config = try_mut_from_ptr!(config);
             config.enable_sni = enable;
         }
     }
 
     /// Provide the configuration a list of certificates where the connection
     /// will select the first one that is compatible with the server's signature
-    /// verification capabilities. Clients that want to support both ECDSA and
-    /// RSA certificates will want the ECSDA to go first in the list.
+    /// verification capabilities.
+    ///
+    /// Clients that want to support both ECDSA and RSA certificates will want the
+    /// ECSDA to go first in the list.
     ///
     /// The built configuration will keep a reference to all certified keys
     /// provided. The client may `rustls_certified_key_free()` afterwards
@@ -459,12 +411,11 @@ impl rustls_client_config_builder {
         certified_keys_len: size_t,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let config: &mut ClientConfigBuilder = try_mut_from_ptr!(builder);
-            let keys_ptrs: &[*const rustls_certified_key] =
-                try_slice!(certified_keys, certified_keys_len);
-            let mut keys: Vec<Arc<CertifiedKey>> = Vec::new();
+            let config = try_mut_from_ptr!(builder);
+            let keys_ptrs = try_slice!(certified_keys, certified_keys_len);
+            let mut keys = Vec::new();
             for &key_ptr in keys_ptrs {
-                let certified_key: Arc<CertifiedKey> = try_clone_arc!(key_ptr);
+                let certified_key = try_clone_arc!(key_ptr);
                 keys.push(certified_key);
             }
             config.cert_resolver = Some(Arc::new(ResolvesClientCertFromChoices { keys }));
@@ -483,8 +434,8 @@ impl ResolvesClientCert for ResolvesClientCertFromChoices {
     fn resolve(
         &self,
         _acceptable_issuers: &[&[u8]],
-        sig_schemes: &[rustls::SignatureScheme],
-    ) -> Option<Arc<rustls::sign::CertifiedKey>> {
+        sig_schemes: &[SignatureScheme],
+    ) -> Option<Arc<CertifiedKey>> {
         for key in self.keys.iter() {
             if key.key.choose_scheme(sig_schemes).is_some() {
                 return Some(key.clone());
@@ -504,26 +455,49 @@ impl rustls_client_config_builder {
     #[no_mangle]
     pub extern "C" fn rustls_client_config_builder_build(
         builder: *mut rustls_client_config_builder,
-    ) -> *const rustls_client_config {
+        config_out: *mut *const rustls_client_config,
+    ) -> rustls_result {
         ffi_panic_boundary! {
-            let builder: Box<ClientConfigBuilder> = try_box_from_ptr!(builder);
-            let config = builder
-                .base
+            let builder = try_box_from_ptr!(builder);
+            let config_out = try_ref_from_ptr_ptr!(config_out);
+
+            let provider = match builder.provider {
+                Some(provider) => provider,
+                None => return rustls_result::NoDefaultCryptoProvider,
+            };
+
+            let verifier = match builder.verifier {
+                Some(v) => v,
+                None => return rustls_result::NoServerCertVerifier,
+            };
+
+            let config = match ClientConfig::builder_with_provider(provider)
+                .with_protocol_versions(&builder.versions)
+            {
+                Ok(c) => c,
+                Err(err) => return map_error(err),
+            };
+
+            let config = config
                 .dangerous()
-                .with_custom_certificate_verifier(builder.verifier);
+                .with_custom_certificate_verifier(verifier);
             let mut config = match builder.cert_resolver {
                 Some(r) => config.with_client_cert_resolver(r),
                 None => config.with_no_client_auth(),
             };
             config.alpn_protocols = builder.alpn_protocols;
             config.enable_sni = builder.enable_sni;
-            to_arc_const_ptr(config)
+
+            set_arc_mut_ptr(config_out, config);
+            rustls_result::Ok
         }
     }
 
     /// "Free" a client_config_builder without building it into a rustls_client_config.
+    ///
     /// Normally builders are built into rustls_client_config via `rustls_client_config_builder_build`
     /// and may not be free'd or otherwise used afterwards.
+    ///
     /// Use free only when the building of a config has to be aborted before a config
     /// was created.
     #[no_mangle]
@@ -535,11 +509,14 @@ impl rustls_client_config_builder {
 }
 
 impl rustls_client_config {
-    /// "Free" a rustls_client_config previously returned from
-    /// rustls_client_config_builder_build. Since rustls_client_config is actually an
-    /// atomically reference-counted pointer, extant client connections may still
-    /// hold an internal reference to the Rust object. However, C code must
-    /// consider this pointer unusable after "free"ing it.
+    /// "Free" a `rustls_client_config` previously returned from
+    /// `rustls_client_config_builder_build`.
+    ///
+    /// Since `rustls_client_config` is actually an atomically reference-counted pointer,
+    /// extant client connections may still hold an internal reference to the Rust object.
+    ///
+    /// However, C code must consider this pointer unusable after "free"ing it.
+    ///
     /// Calling with NULL is fine. Must not be called twice with the same value.
     #[no_mangle]
     pub extern "C" fn rustls_client_config_free(config: *const rustls_client_config) {
@@ -549,11 +526,14 @@ impl rustls_client_config {
     }
 
     /// Create a new rustls_connection containing a client connection and return
-    /// it in the output parameter `out`. If this returns an error code, the
-    /// memory pointed to by `conn_out` remains unchanged. If this returns a
-    /// non-error, the memory pointed to by `conn_out` is modified to point at a
-    /// valid rustls_connection. The caller now owns the rustls_connection and must
-    /// call `rustls_connection_free` when done with it.
+    /// it in the output parameter `conn_out`.
+    ///
+    /// If this returns an error code, the memory pointed to by `conn_out` remains
+    /// unchanged.
+    ///
+    /// If this returns a non-error, the memory pointed to by `conn_out`
+    /// is modified to point at a valid `rustls_connection`.  The caller now owns
+    /// the `rustls_connection` and must call `rustls_connection_free` when done with it.
     ///
     /// The server_name parameter can contain a hostname or an IP address in
     /// textual form (IPv4 or IPv6). This function will return an error if it
@@ -568,18 +548,19 @@ impl rustls_client_config {
             if conn_out.is_null() {
                 return NullParameter;
             }
-            let server_name: &CStr = unsafe {
+            let server_name = unsafe {
                 if server_name.is_null() {
                     return NullParameter;
                 }
                 CStr::from_ptr(server_name)
             };
-            let config: Arc<ClientConfig> = try_clone_arc!(config);
-            let server_name: &str = match server_name.to_str() {
+            let config = try_clone_arc!(config);
+            let conn_out = try_mut_from_ptr_ptr!(conn_out);
+            let server_name = match server_name.to_str() {
                 Ok(s) => s,
                 Err(std::str::Utf8Error { .. }) => return rustls_result::InvalidDnsNameError,
             };
-            let server_name: pki_types::ServerName = match server_name.try_into() {
+            let server_name = match server_name.try_into() {
                 Ok(sn) => sn,
                 Err(_) => return rustls_result::InvalidDnsNameError,
             };
@@ -603,34 +584,57 @@ mod tests {
 
     #[test]
     fn test_config_builder() {
-        let builder: *mut rustls_client_config_builder =
-            rustls_client_config_builder::rustls_client_config_builder_new();
+        let builder = rustls_client_config_builder::rustls_client_config_builder_new();
+        let mut verifier = null_mut();
+        let result =
+            rustls_server_cert_verifier::rustls_platform_server_cert_verifier(&mut verifier);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!verifier.is_null());
+        rustls_client_config_builder::rustls_client_config_builder_set_server_verifier(
+            builder, verifier,
+        );
         let h1 = "http/1.1".as_bytes();
         let h2 = "h2".as_bytes();
-        let alpn: Vec<rustls_slice_bytes> = vec![h1.into(), h2.into()];
+        let alpn = [h1.into(), h2.into()];
         rustls_client_config_builder::rustls_client_config_builder_set_alpn_protocols(
             builder,
             alpn.as_ptr(),
             alpn.len(),
         );
         rustls_client_config_builder::rustls_client_config_builder_set_enable_sni(builder, false);
-        let config = rustls_client_config_builder::rustls_client_config_builder_build(builder);
+        let mut config = null();
+        let result =
+            rustls_client_config_builder::rustls_client_config_builder_build(builder, &mut config);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!config.is_null());
         {
             let config2 = try_ref_from_ptr!(config);
-            assert_eq!(config2.enable_sni, false);
+            assert!(!config2.enable_sni);
             assert_eq!(config2.alpn_protocols, vec![h1, h2]);
         }
-        rustls_client_config::rustls_client_config_free(config)
+        rustls_client_config::rustls_client_config_free(config);
+        rustls_server_cert_verifier::rustls_server_cert_verifier_free(verifier);
     }
 
     // Build a client connection and test the getters and initial values.
     #[test]
     #[cfg_attr(miri, ignore)]
     fn test_client_connection_new() {
-        let builder: *mut rustls_client_config_builder =
-            rustls_client_config_builder::rustls_client_config_builder_new();
-        let config = rustls_client_config_builder::rustls_client_config_builder_build(builder);
-        let mut conn: *mut rustls_connection = null_mut();
+        let builder = rustls_client_config_builder::rustls_client_config_builder_new();
+        let mut verifier = null_mut();
+        let result =
+            rustls_server_cert_verifier::rustls_platform_server_cert_verifier(&mut verifier);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!verifier.is_null());
+        rustls_client_config_builder::rustls_client_config_builder_set_server_verifier(
+            builder, verifier,
+        );
+        let mut config = null();
+        let result =
+            rustls_client_config_builder::rustls_client_config_builder_build(builder, &mut config);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!config.is_null());
+        let mut conn = null_mut();
         let result = rustls_client_config::rustls_client_connection_new(
             config,
             "example.com\0".as_ptr() as *const c_char,
@@ -639,16 +643,13 @@ mod tests {
         if !matches!(result, rustls_result::Ok) {
             panic!("expected RUSTLS_RESULT_OK, got {:?}", result);
         }
-        assert_eq!(rustls_connection::rustls_connection_wants_read(conn), false);
-        assert_eq!(rustls_connection::rustls_connection_wants_write(conn), true);
-        assert_eq!(
-            rustls_connection::rustls_connection_is_handshaking(conn),
-            true
-        );
+        assert!(!rustls_connection::rustls_connection_wants_read(conn));
+        assert!(rustls_connection::rustls_connection_wants_write(conn));
+        assert!(rustls_connection::rustls_connection_is_handshaking(conn));
 
         let some_byte = 42u8;
         let mut alpn_protocol: *const u8 = &some_byte;
-        let mut alpn_protocol_len: usize = 1;
+        let mut alpn_protocol_len = 1;
         rustls_connection::rustls_connection_get_alpn_protocol(
             conn,
             &mut alpn_protocol,
@@ -659,8 +660,10 @@ mod tests {
 
         assert_eq!(
             rustls_connection::rustls_connection_get_negotiated_ciphersuite(conn),
-            null()
+            0
         );
+        let cs_name = rustls_connection::rustls_connection_get_negotiated_ciphersuite_name(conn);
+        assert_eq!(unsafe { cs_name.to_str() }, "");
         assert_eq!(
             rustls_connection::rustls_connection_get_peer_certificate(conn, 0),
             null()
@@ -671,15 +674,27 @@ mod tests {
             0
         );
         rustls_connection::rustls_connection_free(conn);
+        rustls_server_cert_verifier::rustls_server_cert_verifier_free(verifier);
     }
 
     #[test]
     #[cfg_attr(miri, ignore)]
     fn test_client_connection_new_ipaddress() {
-        let builder: *mut rustls_client_config_builder =
-            rustls_client_config_builder::rustls_client_config_builder_new();
-        let config = rustls_client_config_builder::rustls_client_config_builder_build(builder);
-        let mut conn: *mut rustls_connection = null_mut();
+        let builder = rustls_client_config_builder::rustls_client_config_builder_new();
+        let mut verifier = null_mut();
+        let result =
+            rustls_server_cert_verifier::rustls_platform_server_cert_verifier(&mut verifier);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!verifier.is_null());
+        rustls_client_config_builder::rustls_client_config_builder_set_server_verifier(
+            builder, verifier,
+        );
+        let mut config = null();
+        let result =
+            rustls_client_config_builder::rustls_client_config_builder_build(builder, &mut config);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!config.is_null());
+        let mut conn = null_mut();
         let result = rustls_client_config::rustls_client_connection_new(
             config,
             "198.51.100.198\0".as_ptr() as *const c_char,
@@ -688,5 +703,15 @@ mod tests {
         if !matches!(result, rustls_result::Ok) {
             panic!("expected RUSTLS_RESULT_OK, got {:?}", result);
         }
+        rustls_server_cert_verifier::rustls_server_cert_verifier_free(verifier);
+    }
+
+    #[test]
+    fn test_client_builder_no_verifier_err() {
+        let builder = rustls_client_config_builder::rustls_client_config_builder_new();
+        let mut config = null();
+        let result =
+            rustls_client_config_builder::rustls_client_config_builder_build(builder, &mut config);
+        assert_eq!(result, rustls_result::NoServerCertVerifier);
     }
 }
