@@ -4,22 +4,20 @@ use std::{ptr::null_mut, slice};
 
 use libc::{size_t, EINVAL, EIO};
 use pki_types::CertificateDer;
-use rustls::crypto::ring::ALL_CIPHER_SUITES;
-use rustls::{ClientConnection, ServerConnection, SupportedCipherSuite};
+use rustls::CipherSuite::TLS_NULL_WITH_NULL_NULL;
+use rustls::{ClientConnection, ServerConnection};
 
+use crate::cipher::rustls_certificate;
+use crate::error::{map_error, rustls_io_result, rustls_result};
 use crate::io::{
-    rustls_write_vectored_callback, CallbackReader, CallbackWriter, ReadCallback,
-    VectoredCallbackWriter, VectoredWriteCallback, WriteCallback,
+    rustls_read_callback, rustls_write_callback, rustls_write_vectored_callback, CallbackReader,
+    CallbackWriter, VectoredCallbackWriter,
 };
 use crate::log::{ensure_log_registered, rustls_log_callback};
-
+use crate::rslice::rustls_str;
 use crate::{
-    cipher::{rustls_certificate, rustls_supported_ciphersuite},
-    error::{map_error, rustls_io_result, rustls_result},
-    ffi_panic_boundary, free_box,
-    io::{rustls_read_callback, rustls_write_callback},
-    try_callback, try_mut_from_ptr, try_ref_from_ptr, try_slice, userdata_push, Castable,
-    OwnershipBox,
+    box_castable, ffi_panic_boundary, free_box, try_callback, try_mut_from_ptr, try_ref_from_ptr,
+    try_slice, try_slice_mut, userdata_push,
 };
 
 use rustls_result::NullParameter;
@@ -93,13 +91,8 @@ impl std::ops::DerefMut for Connection {
     }
 }
 
-pub struct rustls_connection {
-    _private: [u8; 0],
-}
-
-impl Castable for rustls_connection {
-    type Ownership = OwnershipBox;
-    type RustType = Connection;
+box_castable! {
+    pub struct rustls_connection(Connection);
 }
 
 impl rustls_connection {
@@ -111,7 +104,7 @@ impl rustls_connection {
         conn: *mut rustls_connection,
         userdata: *mut c_void,
     ) {
-        let conn: &mut Connection = try_mut_from_ptr!(conn);
+        let conn = try_mut_from_ptr!(conn);
         conn.userdata = userdata;
     }
 
@@ -123,7 +116,7 @@ impl rustls_connection {
         conn: *mut rustls_connection,
         cb: rustls_log_callback,
     ) {
-        let conn: &mut Connection = try_mut_from_ptr!(conn);
+        let conn = try_mut_from_ptr!(conn);
         ensure_log_registered();
         conn.log_callback = cb;
     }
@@ -146,14 +139,14 @@ impl rustls_connection {
         out_n: *mut size_t,
     ) -> rustls_io_result {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
+            let conn = try_mut_from_ptr!(conn);
             if out_n.is_null() {
                 return rustls_io_result(EINVAL);
             }
-            let callback: ReadCallback = try_callback!(callback);
+            let callback = try_callback!(callback);
 
             let mut reader = CallbackReader { callback, userdata };
-            let n_read: usize = match conn.read_tls(&mut reader) {
+            let n_read = match conn.read_tls(&mut reader) {
                 Ok(n) => n,
                 Err(e) => return rustls_io_result(e.raw_os_error().unwrap_or(EIO)),
             };
@@ -183,14 +176,14 @@ impl rustls_connection {
         out_n: *mut size_t,
     ) -> rustls_io_result {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
+            let conn = try_mut_from_ptr!(conn);
             if out_n.is_null() {
                 return rustls_io_result(EINVAL);
             }
-            let callback: WriteCallback = try_callback!(callback);
+            let callback = try_callback!(callback);
 
             let mut writer = CallbackWriter { callback, userdata };
-            let n_written: usize = match conn.write_tls(&mut writer) {
+            let n_written = match conn.write_tls(&mut writer) {
                 Ok(n) => n,
                 Err(e) => return rustls_io_result(e.raw_os_error().unwrap_or(EIO)),
             };
@@ -220,14 +213,14 @@ impl rustls_connection {
         out_n: *mut size_t,
     ) -> rustls_io_result {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
+            let conn = try_mut_from_ptr!(conn);
             if out_n.is_null() {
                 return rustls_io_result(EINVAL);
             }
-            let callback: VectoredWriteCallback = try_callback!(callback);
+            let callback = try_callback!(callback);
 
             let mut writer = VectoredCallbackWriter { callback, userdata };
-            let n_written: usize = match conn.write_tls(&mut writer) {
+            let n_written = match conn.write_tls(&mut writer) {
                 Ok(n) => n,
                 Err(e) => return rustls_io_result(e.raw_os_error().unwrap_or(EIO)),
             };
@@ -248,7 +241,7 @@ impl rustls_connection {
         conn: *mut rustls_connection,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
+            let conn = try_mut_from_ptr!(conn);
             let guard = match userdata_push(conn.userdata, conn.log_callback) {
                 Ok(g) => g,
                 Err(_) => return rustls_result::Panic,
@@ -268,7 +261,7 @@ impl rustls_connection {
     #[no_mangle]
     pub extern "C" fn rustls_connection_wants_read(conn: *const rustls_connection) -> bool {
         ffi_panic_boundary! {
-            let conn: &Connection = try_ref_from_ptr!(conn);
+            let conn = try_ref_from_ptr!(conn);
             conn.wants_read()
         }
     }
@@ -277,16 +270,23 @@ impl rustls_connection {
     #[no_mangle]
     pub extern "C" fn rustls_connection_wants_write(conn: *const rustls_connection) -> bool {
         ffi_panic_boundary! {
-            let conn: &Connection = try_ref_from_ptr!(conn);
+            let conn = try_ref_from_ptr!(conn);
             conn.wants_write()
         }
     }
 
+    /// Returns true if the connection is currently performing the TLS handshake.
+    ///
+    /// Note: This may return `false` while there are still handshake packets waiting
+    /// to be extracted and transmitted with `rustls_connection_write_tls()`.
+    ///
+    /// See the rustls documentation for more information.
+    ///
     /// <https://docs.rs/rustls/latest/rustls/struct.CommonState.html#method.is_handshaking>
     #[no_mangle]
     pub extern "C" fn rustls_connection_is_handshaking(conn: *const rustls_connection) -> bool {
         ffi_panic_boundary! {
-            let conn: &Connection = try_ref_from_ptr!(conn);
+            let conn = try_ref_from_ptr!(conn);
             conn.is_handshaking()
         }
     }
@@ -299,7 +299,7 @@ impl rustls_connection {
     #[no_mangle]
     pub extern "C" fn rustls_connection_set_buffer_limit(conn: *mut rustls_connection, n: usize) {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
+            let conn = try_mut_from_ptr!(conn);
             conn.set_buffer_limit(Some(n));
         }
     }
@@ -309,7 +309,7 @@ impl rustls_connection {
     #[no_mangle]
     pub extern "C" fn rustls_connection_send_close_notify(conn: *mut rustls_connection) {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
+            let conn = try_mut_from_ptr!(conn);
             conn.send_close_notify();
         }
     }
@@ -329,7 +329,7 @@ impl rustls_connection {
         i: size_t,
     ) -> *const rustls_certificate<'a> {
         ffi_panic_boundary! {
-            let conn: &Connection = try_ref_from_ptr!(conn);
+            let conn = try_ref_from_ptr!(conn);
             match conn.peer_certificates().and_then(|c| c.get(i)) {
                 Some(cert) => cert as *const CertificateDer as *const _,
                 None => null(),
@@ -355,7 +355,7 @@ impl rustls_connection {
         protocol_out_len: *mut usize,
     ) {
         ffi_panic_boundary! {
-            let conn: &Connection = try_ref_from_ptr!(conn);
+            let conn = try_ref_from_ptr!(conn);
             if protocol_out.is_null() || protocol_out_len.is_null() {
                 return;
             }
@@ -382,7 +382,7 @@ impl rustls_connection {
         conn: *const rustls_connection,
     ) -> u16 {
         ffi_panic_boundary! {
-            let conn: &Connection = try_ref_from_ptr!(conn);
+            let conn = try_ref_from_ptr!(conn);
             match conn.protocol_version() {
                 Some(p) => u16::from(p),
                 _ => 0,
@@ -390,30 +390,44 @@ impl rustls_connection {
         }
     }
 
-    /// Retrieves the cipher suite agreed with the peer.
-    /// This returns NULL until the ciphersuite is agreed.
-    /// The returned pointer lives as long as the program.
-    /// <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.negotiated_cipher_suite>
+    /// Retrieves the [IANA registered cipher suite identifier][IANA] agreed with the peer.
+    ///
+    /// This returns `TLS_NULL_WITH_NULL_NULL` (0x0000) until the ciphersuite is agreed.
+    ///
+    /// [IANA]: <https://www.iana.org/assignments/tls-parameters/tls-parameters.xhtml#tls-parameters-4>
     #[no_mangle]
     pub extern "C" fn rustls_connection_get_negotiated_ciphersuite(
         conn: *const rustls_connection,
-    ) -> *const rustls_supported_ciphersuite {
+    ) -> u16 {
         ffi_panic_boundary! {
-            let conn: &Connection = try_ref_from_ptr!(conn);
-            let negotiated = match conn.negotiated_cipher_suite() {
+            match try_ref_from_ptr!(conn).negotiated_cipher_suite() {
+                Some(cs) => u16::from(cs.suite()),
+                None => u16::from(TLS_NULL_WITH_NULL_NULL),
+            }
+        }
+    }
+
+    /// Retrieves the cipher suite name agreed with the peer.
+    ///
+    /// This returns "" until the ciphersuite is agreed.
+    ///
+    /// The lifetime of the `rustls_str` is the lifetime of the program, it does not
+    /// need to be freed.
+    ///
+    /// <https://docs.rs/rustls/latest/rustls/enum.Connection.html#method.negotiated_cipher_suite>
+    #[no_mangle]
+    pub extern "C" fn rustls_connection_get_negotiated_ciphersuite_name(
+        conn: *const rustls_connection,
+    ) -> rustls_str<'static> {
+        ffi_panic_boundary! {
+            let cs_name = try_ref_from_ptr!(conn)
+                .negotiated_cipher_suite()
+                .and_then(|cs| cs.suite().as_str())
+                .and_then(|name| rustls_str::try_from(name).ok());
+            match cs_name {
                 Some(cs) => cs,
-                None => return null(),
-            };
-            for cs in ALL_CIPHER_SUITES {
-                // This type annotation is here to enforce the lifetime stated
-                // in the doccomment - that the returned pointer lives as long
-                // as the program.
-                let cs: &'static SupportedCipherSuite = cs;
-                if negotiated == *cs {
-                    return cs as *const SupportedCipherSuite as *const _;
-                }
+                None => rustls_str::from_str_unchecked(""),
             }
-            null()
         }
     }
 
@@ -431,12 +445,12 @@ impl rustls_connection {
         out_n: *mut size_t,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
-            let write_buf: &[u8] = try_slice!(buf, count);
+            let conn = try_mut_from_ptr!(conn);
+            let write_buf = try_slice!(buf, count);
             if out_n.is_null() {
                 return NullParameter;
             }
-            let n_written: usize = match conn.writer().write(write_buf) {
+            let n_written = match conn.writer().write(write_buf) {
                 Ok(n) => n,
                 Err(_) => return rustls_result::Io,
             };
@@ -467,7 +481,7 @@ impl rustls_connection {
         out_n: *mut size_t,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
+            let conn = try_mut_from_ptr!(conn);
             if buf.is_null() {
                 return NullParameter;
             }
@@ -477,9 +491,9 @@ impl rustls_connection {
 
             // Safety: the memory pointed at by buf must be initialized
             // (required by documentation of this function).
-            let read_buf: &mut [u8] = unsafe { slice::from_raw_parts_mut(buf, count) };
+            let read_buf = try_slice_mut!(buf, count);
 
-            let n_read: usize = match conn.reader().read(read_buf) {
+            let n_read = match conn.reader().read(read_buf) {
                 Ok(n) => n,
                 Err(e) if e.kind() == ErrorKind::UnexpectedEof => {
                     return rustls_result::UnexpectedEof
@@ -518,16 +532,13 @@ impl rustls_connection {
         out_n: *mut size_t,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let conn: &mut Connection = try_mut_from_ptr!(conn);
+            let conn = try_mut_from_ptr!(conn);
             if buf.is_null() || out_n.is_null() {
                 return NullParameter;
             }
-            let read_buf: &mut [std::mem::MaybeUninit<u8>] =
-                unsafe { slice::from_raw_parts_mut(buf, count) };
+            let mut read_buf: std::io::BorrowedBuf<'_> = try_slice_mut!(buf, count).into();
 
-            let mut read_buf: std::io::BorrowedBuf<'_> = read_buf.into();
-
-            let n_read: usize = match conn.reader().read_buf(read_buf.unfilled()) {
+            let n_read = match conn.reader().read_buf(read_buf.unfilled()) {
                 Ok(()) => read_buf.filled().len(),
                 Err(e) if e.kind() == ErrorKind::UnexpectedEof => {
                     return rustls_result::UnexpectedEof
