@@ -1,7 +1,11 @@
+use rustls::{ProtocolVersion, SupportedProtocolVersion};
+
+#[derive(Debug, Default)]
 #[repr(C)]
-#[allow(dead_code)]
 /// Definitions of known TLS protocol versions.
 pub enum rustls_tls_version {
+    #[default]
+    Unknown = 0x0000,
     Sslv2 = 0x0200,
     Sslv3 = 0x0300,
     Tlsv1_0 = 0x0301,
@@ -10,6 +14,20 @@ pub enum rustls_tls_version {
     Tlsv1_3 = 0x0304,
 }
 
+impl From<&SupportedProtocolVersion> for rustls_tls_version {
+    fn from(version: &SupportedProtocolVersion) -> Self {
+        match version.version {
+            ProtocolVersion::SSLv2 => rustls_tls_version::Sslv2,
+            ProtocolVersion::SSLv3 => rustls_tls_version::Sslv3,
+            ProtocolVersion::TLSv1_0 => rustls_tls_version::Tlsv1_0,
+            ProtocolVersion::TLSv1_1 => rustls_tls_version::Tlsv1_1,
+            ProtocolVersion::TLSv1_2 => rustls_tls_version::Tlsv1_2,
+            ProtocolVersion::TLSv1_3 => rustls_tls_version::Tlsv1_3,
+            _ => rustls_tls_version::Unknown,
+        }
+    }
+}
+
 /// Rustls' list of supported protocol versions. The length of the array is
 /// given by `RUSTLS_ALL_VERSIONS_LEN`.
 #[no_mangle]
