@@ -8,9 +8,10 @@ use rustls::server::VerifierBuilderError;
 use rustls::{CertRevocationListError, CertificateError, Error, InvalidMessage};
 
 /// A return value for a function that may return either success (0) or a
-/// non-zero value representing an error. The values should match socket
-/// error numbers for your operating system - for example, the integers for
-/// ETIMEDOUT, EAGAIN, or similar.
+/// non-zero value representing an error.
+///
+/// The values should match socket error numbers for your operating system --
+/// for example, the integers for `ETIMEDOUT`, `EAGAIN`, or similar.
 #[repr(transparent)]
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct rustls_io_result(pub libc::c_int);
@@ -59,6 +60,9 @@ u32_enum_builder! {
         AcceptorNotReady => 7012,
         AlreadyUsed => 7013,
         CertificateRevocationListParseError => 7014,
+        NoServerCertVerifier => 7015,
+        NoDefaultCryptoProvider => 7016,
+        GetRandomFailed => 7017,
 
         // From https://docs.rs/rustls/latest/rustls/enum.Error.html
         NoCertificatesPresented => 7101,
@@ -93,6 +97,7 @@ u32_enum_builder! {
         CertInvalidPurpose => 7129,
         CertApplicationVerificationFailure => 7130,
         CertOtherError => 7131,
+        CertUnknownRevocationStatus => 7154, // Last added.
 
         // From InvalidMessage, with fields that get flattened.
         // https://docs.rs/rustls/0.21.0/rustls/enum.Error.html#variant.InvalidMessage
@@ -116,7 +121,7 @@ u32_enum_builder! {
         MessageUnsupportedCompression => 7150,
         MessageUnsupportedCurveType => 7151,
         MessageUnsupportedKeyExchangeAlgorithm => 7152,
-        MessageInvalidOther => 7153, // Last added.
+        MessageInvalidOther => 7153,
 
         // From Error, with fields that get dropped.
         PeerIncompatibleError => 7107,
@@ -192,7 +197,9 @@ u32_enum_builder! {
 impl rustls_result {
     /// After a rustls function returns an error, you may call
     /// this to get a pointer to a buffer containing a detailed error
-    /// message. The contents of the error buffer will be out_n bytes long,
+    /// message.
+    ///
+    /// The contents of the error buffer will be out_n bytes long,
     /// UTF-8 encoded, and not NUL-terminated.
     #[no_mangle]
     pub extern "C" fn rustls_error(
@@ -209,7 +216,7 @@ impl rustls_result {
                 return;
             }
             let error_str = rustls_result::from(result).to_string();
-            let out_len: usize = min(len - 1, error_str.len());
+            let out_len = min(len - 1, error_str.len());
             unsafe {
                 std::ptr::copy_nonoverlapping(error_str.as_ptr() as *mut c_char, buf, out_len);
                 *out_n = out_len;
@@ -228,6 +235,7 @@ impl rustls_result {
                 | CertRevoked
                 | CertUnhandledCriticalExtension
                 | CertUnknownIssuer
+                | CertUnknownRevocationStatus
                 | CertBadSignature
                 | CertNotValidForName
                 | CertInvalidPurpose
@@ -237,9 +245,10 @@ impl rustls_result {
     }
 }
 
-/// For cert-related rustls_results, turn them into a rustls::Error. For other
-/// inputs, including Ok, return rustls::Error::General.
-pub(crate) fn cert_result_to_error(result: rustls_result) -> rustls::Error {
+/// For cert-related rustls_results, turn them into a rustls::Error.
+///
+/// For other inputs, including Ok, return rustls::Error::General.
+pub(crate) fn cert_result_to_error(result: rustls_result) -> Error {
     use rustls::Error::*;
     use rustls::OtherError;
     use rustls_result::*;
@@ -270,15 +279,15 @@ fn test_rustls_error() {
     let mut buf = [0 as c_char; 512];
     let mut n = 0;
     rustls_result::rustls_error(0, &mut buf as *mut _, buf.len(), &mut n);
-    let output: String = String::from_utf8(buf[0..n].iter().map(|b| *b as u8).collect()).unwrap();
+    let output = String::from_utf8(buf[0..n].iter().map(|b| *b as u8).collect()).unwrap();
     assert_eq!(&output, "a parameter had an invalid value");
 
     rustls_result::rustls_error(7000, &mut buf as *mut _, buf.len(), &mut n);
-    let output: String = String::from_utf8(buf[0..n].iter().map(|b| *b as u8).collect()).unwrap();
+    let output = String::from_utf8(buf[0..n].iter().map(|b| *b as u8).collect()).unwrap();
     assert_eq!(&output, "OK");
 
     rustls_result::rustls_error(7101, &mut buf as *mut _, buf.len(), &mut n);
-    let output: String = String::from_utf8(buf[0..n].iter().map(|b| *b as u8).collect()).unwrap();
+    let output = String::from_utf8(buf[0..n].iter().map(|b| *b as u8).collect()).unwrap();
     assert_eq!(&output, "peer sent no certificates");
 }
 
@@ -293,7 +302,7 @@ fn test_rustls_result_is_cert_error() {
     }
 }
 
-pub(crate) fn map_error(input: rustls::Error) -> rustls_result {
+pub(crate) fn map_error(input: Error) -> rustls_result {
     use rustls::AlertDescription as alert;
     use rustls_result::*;
 
@@ -346,6 +355,7 @@ pub(crate) fn map_error(input: rustls::Error) -> rustls_result {
             CertificateError::Revoked => CertRevoked,
             CertificateError::UnhandledCriticalExtension => CertUnhandledCriticalExtension,
             CertificateError::UnknownIssuer => CertUnknownIssuer,
+            CertificateError::UnknownRevocationStatus => CertUnknownRevocationStatus,
             CertificateError::BadSignature => CertBadSignature,
             CertificateError::NotValidForName => CertNotValidForName,
             CertificateError::InvalidPurpose => CertInvalidPurpose,
@@ -474,6 +484,21 @@ impl Display for rustls_result {
             CertificateRevocationListParseError => {
                 write!(f, "error parsing certificate revocation list (CRL)",)
             }
+            NoServerCertVerifier => {
+                write!(
+                    f,
+                    "no server certificate verifier was configured on the client config builder"
+                )
+            }
+            NoDefaultCryptoProvider => {
+                write!(
+                    f,
+                    "no default process-wide crypto provider has been installed"
+                )
+            }
+            GetRandomFailed => {
+                write!(f, "failed to get random bytes from the crypto provider")
+            }
 
             CertEncodingBad => Error::InvalidCertificate(CertificateError::BadEncoding).fmt(f),
             CertExpired => Error::InvalidCertificate(CertificateError::Expired).fmt(f),
@@ -493,6 +518,9 @@ impl Display for rustls_result {
             CertApplicationVerificationFailure => {
                 Error::InvalidCertificate(CertificateError::ApplicationVerificationFailure).fmt(f)
             }
+            CertUnknownRevocationStatus => {
+                Error::InvalidCertificate(CertificateError::UnknownRevocationStatus).fmt(f)
+            }
             CertOtherError => write!(f, "unknown certificate error"),
 
             // These variants correspond to a rustls::Error variant with a field,
