@@ -1,5 +1,7 @@
 #![crate_type = "staticlib"]
 #![allow(non_camel_case_types)]
+// TODO(XXX): Remove `renamed_and_removed_lints` once stable renames `thread_local_initializer_can_be_made_const`
+#![allow(renamed_and_removed_lints)]
 #![allow(clippy::not_unsafe_ptr_arg_deref)]
 // TODO(#333): Fix this clippy warning.
 #![allow(clippy::arc_with_non_send_sync)]
@@ -25,6 +27,7 @@ pub mod acceptor;
 pub mod cipher;
 pub mod client;
 pub mod connection;
+pub mod crypto_provider;
 pub mod enums;
 mod error;
 pub mod io;
@@ -54,21 +57,27 @@ include!(concat!(env!("OUT_DIR"), "/version.rs"));
 // Rust code, we model these thread locals as a stack, so we can always
 // restore the previous version.
 thread_local! {
-    #[allow(clippy::thread_local_initializer_can_be_made_const)]
+    // TODO(XXX): Remove 'thread_local_initializer_can_be_made_const' in the future
+    //            once stable has renamed.
+    #[allow(clippy::thread_local_initializer_can_be_made_const, clippy::missing_const_for_thread_local)]
     pub(crate) static USERDATA: RefCell<Vec<Userdata>> = RefCell::new(Vec::new());
 }
 
 pub(crate) struct Userdata {
     userdata: *mut c_void,
+    #[cfg(not(feature = "no_log_capture"))]
     log_callback: rustls_log_callback,
 }
 
 /// UserdataGuard pops an entry off the USERDATA stack, restoring the
 /// thread-local state to its value previous to the creation of the UserdataGuard.
+///
 /// Invariants: As long as a UserdataGuard is live:
+///
 ///  - The stack of userdata items for this thread must have at least one item.
 ///  - The top item on that stack must be the one this guard was built with.
 ///  - The `data` field must not be None.
+///
 /// If any of these invariants fails, try_drop will return an error.
 pub(crate) struct UserdataGuard {
     // Keep a copy of the data we expect to be popping off the stack. This allows
@@ -82,6 +91,7 @@ impl UserdataGuard {
         UserdataGuard {
             data: Some(Userdata {
                 userdata: u,
+                #[cfg(not(feature = "no_log_capture"))]
                 log_callback: None,
             }),
         }
@@ -144,7 +154,7 @@ pub(crate) enum UserdataError {
 #[must_use = "If you drop the guard, userdata will be immediately cleared"]
 pub(crate) fn userdata_push(
     u: *mut c_void,
-    cb: rustls_log_callback,
+    _cb: rustls_log_callback,
 ) -> Result<UserdataGuard, UserdataError> {
     USERDATA
         .try_with(|userdata| {
@@ -153,7 +163,8 @@ pub(crate) fn userdata_push(
                 |mut v| {
                     v.push(Userdata {
                         userdata: u,
-                        log_callback: cb,
+                        #[cfg(not(feature = "no_log_capture"))]
+                        log_callback: _cb,
                     });
                     Ok(())
                 },
@@ -177,6 +188,7 @@ pub(crate) fn userdata_get() -> Result<*mut c_void, UserdataError> {
         .unwrap_or(Err(UserdataError::AccessError))
 }
 
+#[cfg(not(feature = "no_log_capture"))]
 pub(crate) fn log_callback_get() -> Result<(rustls_log_callback, *mut c_void), UserdataError> {
     USERDATA
         .try_with(|userdata| {
@@ -191,109 +203,6 @@ pub(crate) fn log_callback_get() -> Result<(rustls_log_callback, *mut c_void), U
         .unwrap_or(Err(UserdataError::AccessError))
 }
 
-#[cfg(test)]
-mod tests {
-    use super::*;
-    use std::thread;
-
-    #[test]
-    fn guard_try_pop() {
-        let data = "hello";
-        let data_ptr: *mut c_void = data as *const _ as _;
-        let mut guard = userdata_push(data_ptr, None).unwrap();
-        assert_eq!(userdata_get().unwrap(), data_ptr);
-        guard.try_pop().unwrap();
-        assert!(matches!(guard.try_pop(), Err(_)));
-    }
-
-    #[test]
-    fn guard_try_drop() {
-        let data = "hello";
-        let data_ptr: *mut c_void = data as *const _ as _;
-        let guard = userdata_push(data_ptr, None).unwrap();
-        assert_eq!(userdata_get().unwrap(), data_ptr);
-        guard.try_drop().unwrap();
-        assert!(matches!(userdata_get(), Err(_)));
-    }
-
-    #[test]
-    fn guard_drop() {
-        let data = "hello";
-        let data_ptr: *mut c_void = data as *const _ as _;
-        {
-            let _guard = userdata_push(data_ptr, None).unwrap();
-            assert_eq!(userdata_get().unwrap(), data_ptr);
-        }
-        assert!(matches!(userdata_get(), Err(_)));
-    }
-
-    #[test]
-    fn nested_guards() {
-        let hello = "hello";
-        let hello_ptr: *mut c_void = hello as *const _ as _;
-        {
-            let guard = userdata_push(hello_ptr, None).unwrap();
-            assert_eq!(userdata_get().unwrap(), hello_ptr);
-            {
-                let yo = "yo";
-                let yo_ptr: *mut c_void = yo as *const _ as _;
-                let guard2 = userdata_push(yo_ptr, None).unwrap();
-                assert_eq!(userdata_get().unwrap(), yo_ptr);
-                guard2.try_drop().unwrap();
-            }
-            assert_eq!(userdata_get().unwrap(), hello_ptr);
-            guard.try_drop().unwrap();
-        }
-        assert!(matches!(userdata_get(), Err(_)));
-    }
-
-    #[test]
-    fn out_of_order_drop() {
-        let hello = "hello";
-        let hello_ptr: *mut c_void = hello as *const _ as _;
-        let guard = userdata_push(hello_ptr, None).unwrap();
-        assert_eq!(userdata_get().unwrap(), hello_ptr);
-
-        let yo = "yo";
-        let yo_ptr: *mut c_void = yo as *const _ as _;
-        let guard2 = userdata_push(yo_ptr, None).unwrap();
-        assert_eq!(userdata_get().unwrap(), yo_ptr);
-
-        assert!(matches!(guard.try_drop(), Err(UserdataError::WrongData)));
-        assert!(matches!(guard2.try_drop(), Err(UserdataError::WrongData)));
-    }
-
-    #[test]
-    fn userdata_multi_threads() {
-        let hello = "hello";
-        let hello_ptr: *mut c_void = hello as *const _ as _;
-        let guard = userdata_push(hello_ptr, None).unwrap();
-        assert_eq!(userdata_get().unwrap(), hello_ptr);
-
-        let thread1 = thread::spawn(|| {
-            let yo = "yo";
-            let yo_ptr: *mut c_void = yo as *const _ as _;
-            let guard2 = userdata_push(yo_ptr, None).unwrap();
-            assert_eq!(userdata_get().unwrap(), yo_ptr);
-
-            let greetz = "greetz";
-            let greetz_ptr: *mut c_void = greetz as *const _ as _;
-
-            let guard3 = userdata_push(greetz_ptr, None).unwrap();
-
-            assert_eq!(userdata_get().unwrap(), greetz_ptr);
-            guard3.try_drop().unwrap();
-
-            assert_eq!(userdata_get().unwrap(), yo_ptr);
-            guard2.try_drop().unwrap();
-        });
-
-        assert_eq!(userdata_get().unwrap(), hello_ptr);
-        guard.try_drop().unwrap();
-        thread1.join().unwrap();
-    }
-}
-
 /// Used to mark that pointer to a [`Castable`]'s underlying `Castable::RustType` is provided
 /// to C code as a pointer to a `Box<Castable::RustType>`.
 pub(crate) struct OwnershipBox;
@@ -339,6 +248,10 @@ impl OwnershipMarker for OwnershipRef {}
 /// preserving const-ness, and casting between the correct types. Implementing this is required in
 /// order to use `try_ref_from_ptr!` or `try_mut_from_ptr!` and several other helpful cast-related
 /// conversion helpers.
+///
+/// You can define a new `Castable` type using the `box_castable!`, `arc_castable!` or
+/// `ref_castable!` macros depending on the ownership marker you desire. See each macro's
+/// documentation for more information.
 pub(crate) trait Castable {
     /// Indicates whether to use `Box` or `Arc` when giving a pointer to C code for the underlying
     /// `RustType`.
@@ -348,6 +261,100 @@ pub(crate) trait Castable {
     type RustType;
 }
 
+/// Defines a new [`Castable`] opaque struct with [`OwnershipBox`] ownership.
+///
+/// Expects to be invoked with a visibility specifier, the struct keyword, a struct name, and
+/// in parens, the Rust type pointed to by raw pointers of the opaque struct's type.
+/// Similar to a [newtype] with `#[repr(transparent)]`, but only allows conversions
+/// between `Box` and raw pointers.
+///
+/// [newtype]: https://doc.rust-lang.org/book/ch19-04-advanced-types.html#using-the-newtype-pattern-for-type-safety-and-abstraction
+macro_rules! box_castable {
+    (
+        $(#[$comment:meta])*
+        pub struct $name:ident($rust_type:ty);
+    ) => {
+        crate::castable!(OwnershipBox $(#[$comment])* $name $rust_type);
+    };
+}
+
+pub(crate) use box_castable;
+
+/// Defines a new [`Castable`] opaque struct with [`OwnershipArc`] ownership.
+///
+/// Expects to be invoked with a visibility specifier, the struct keyword, a struct name, and
+/// in parens, the Rust type pointed to by raw pointers of the opaque struct's type.
+/// Similar to a [newtype] with `#[repr(transparent)]`, but only allows conversions
+/// between `Arc` and raw pointers.
+///
+/// [newtype]: https://doc.rust-lang.org/book/ch19-04-advanced-types.html#using-the-newtype-pattern-for-type-safety-and-abstraction
+macro_rules! arc_castable {
+    (
+        $(#[$comment:meta])*
+        pub struct $name:ident($rust_type:ty);
+    ) => {
+        crate::castable!(OwnershipArc $(#[$comment])* $name $rust_type);
+    };
+}
+
+pub(crate) use arc_castable;
+
+/// Defines a new [`Castable`] opaque struct with [`OwnershipRef`] ownership.
+///
+/// Expects to be invoked with a visibility specifier, the struct keyword, a struct name, and
+/// in parens, the Rust type pointed to by raw pointers of the opaque struct's type.
+/// Similar to a [newtype] with `#[repr(transparent)]`, but only allows conversions
+/// between a reference and raw pointers.
+///
+/// If a lifetime parameter is specified, the opaque struct will be parameterized by it,
+/// and a `PhantomData` field referencing the lifetime is added to the struct.
+///
+/// [newtype]: https://doc.rust-lang.org/book/ch19-04-advanced-types.html#using-the-newtype-pattern-for-type-safety-and-abstraction
+macro_rules! ref_castable {
+    (
+        $(#[$comment:meta])*
+        pub struct $name:ident ($rust_type:ident $(<$lt:tt>)?);
+    ) => {
+        $(#[$comment])*
+        pub struct $name $(<$lt>)? {
+            _private: [u8; 0],
+            $( _marker: PhantomData<&$lt ()>, )?
+        }
+
+        impl $(<$lt>)? crate::Castable for $name $(<$lt>)? {
+            type Ownership = crate::OwnershipRef;
+            type RustType = $rust_type $(<$lt>)?;
+        }
+    };
+}
+
+pub(crate) use ref_castable;
+
+/// Defines a new [`Castable`] opaque struct with the specified ownership.
+///
+/// In general you should prefer using `box_castable!`, `arc_castable!`, or `ref_castable!`
+/// instead of this macro.
+macro_rules! castable {
+    (
+        $ownership:ident
+        $(#[$comment:meta])*
+        $name:ident
+        $rust_type:ty
+    ) => {
+        $(#[$comment])*
+        pub struct $name {
+            _private: [u8; 0],
+        }
+
+        impl crate::Castable for $name {
+            type Ownership = crate::$ownership;
+            type RustType = $rust_type;
+        }
+    };
+}
+
+pub(crate) use castable;
+
 /// Convert a const pointer to a [`Castable`] to a const pointer to its underlying
 /// [`Castable::RustType`].
 ///
@@ -485,17 +492,11 @@ where
 /// Converts a [`Castable`]'s underlying [`Castable::RustType`] to a mutable pointer
 /// to a `Box` over the rust type and sets the `dst` out pointer to the resulting mutable `Box`
 /// pointer. See [`to_boxed_mut_ptr`] for more information.
-///
-/// ## Unsafety:
-///
-/// `dst` must not be `NULL`.
-pub(crate) fn set_boxed_mut_ptr<C>(dst: *mut *mut C, src: C::RustType)
+pub(crate) fn set_boxed_mut_ptr<C>(dst: &mut *mut C, src: C::RustType)
 where
     C: Castable<Ownership = OwnershipBox>,
 {
-    unsafe {
-        *dst = to_boxed_mut_ptr(src);
-    }
+    *dst = to_boxed_mut_ptr(src);
 }
 
 /// Converts a [`Castable`]'s underlying [`Castable::RustType`] to a const pointer
@@ -505,13 +506,11 @@ where
 /// ## Unsafety:
 ///
 /// `dst` must not be `NULL`.
-pub(crate) fn set_arc_mut_ptr<C>(dst: *mut *const C, src: C::RustType)
+pub(crate) fn set_arc_mut_ptr<C>(dst: &mut *const C, src: C::RustType)
 where
     C: Castable<Ownership = OwnershipArc>,
 {
-    unsafe {
-        *dst = to_arc_const_ptr(src);
-    }
+    *dst = to_arc_const_ptr(src);
 }
 
 /// Converts a mutable pointer to a [`Castable`] to an optional ref to the underlying
@@ -540,6 +539,32 @@ macro_rules! try_mut_from_ptr {
 
 pub(crate) use try_mut_from_ptr;
 
+/// Converts a mutable pointer to a mutable pointer to a [`Castable`] to an optional mutable ref to
+/// the mutable pointer to the  [`Castable::RustType`].
+///
+/// Does nothing, returning `None`, when passed `NULL`.
+pub(crate) fn try_from_mut_mut<'a, C: Castable>(from: *mut *mut C) -> Option<&'a mut *mut C> {
+    match from.is_null() {
+        true => None,
+        false => unsafe { Some(&mut *from) },
+    }
+}
+
+/// If the provided pointer to a pointer to a [`Castable`] is non-null, convert it to a mutable
+/// reference to a pointer using [`try_from_mut_mut`]. Otherwise, return
+/// [`rustls_result::NullParameter`], or an appropriate default (`false`, `0`, `NULL`) based on the
+/// context. See [`try_from_mut_mut`] for more information.
+macro_rules! try_mut_from_ptr_ptr {
+    ( $var:ident ) => {
+        match $crate::try_from_mut_mut($var) {
+            Some(c) => c,
+            None => return $crate::panic::NullParameterOrDefault::value(),
+        }
+    };
+}
+
+pub(crate) use try_mut_from_ptr_ptr;
+
 /// Converts a const pointer to a [`Castable`] to an optional ref to the underlying
 /// [`Castable::RustType`]. See [`cast_const_ptr`] for more information.
 ///
@@ -569,10 +594,41 @@ macro_rules! try_ref_from_ptr {
 
 pub(crate) use try_ref_from_ptr;
 
+/// Converts a mut pointer to a const pointer to a [`Castable`] to an optional mut ref to the
+/// const pointer to the underlying [`Castable::RustType`].
+///
+/// Does nothing, returning `None` when passed `NULL`.
+pub(crate) fn try_from_ptr<'a, C>(from: *mut *const C) -> Option<&'a mut *const C>
+where
+    C: Castable,
+{
+    match from.is_null() {
+        true => None,
+        false => unsafe { Some(&mut *from) },
+    }
+}
+
+/// If the provided pointer to pointer to a [`Castable`] is non-null, convert it to a mutable
+/// reference to a pointer to the [`Castable`] using
+/// [`try_from_ptr`]. Otherwise, return [`rustls_result::NullParameter`], or an appropriate default
+/// (`false`, `0`, `NULL`) based on the context;
+///
+/// See [`try_from_ptr`] for more information.
+macro_rules! try_ref_from_ptr_ptr {
+    ( $var:ident ) => {
+        match $crate::try_from_ptr($var) {
+            Some(c) => c,
+            None => return $crate::panic::NullParameterOrDefault::value(),
+        }
+    };
+}
+
+pub(crate) use try_ref_from_ptr_ptr;
+
 /// If the provided pointer to a [`Castable`] is non-null, convert it to a reference to an `Arc` over
-/// the underlying rust type using [`try_arc_from`]. Otherwise, return
+/// the underlying rust type using [`clone_arc`]. Otherwise, return
 /// [`rustls_result::NullParameter`], or an appropriate default (`false`, `0`, `NULL`) based on the
-/// context. See [`try_arc_from`] for more information.
+/// context. See [`clone_arc`] for more information.
 macro_rules! try_clone_arc {
     ( $var:ident ) => {
         match $crate::clone_arc($var) {
@@ -616,13 +672,25 @@ macro_rules! try_slice {
         if $ptr.is_null() {
             return $crate::panic::NullParameterOrDefault::value();
         } else {
-            unsafe { slice::from_raw_parts($ptr, $count as usize) }
+            unsafe { slice::from_raw_parts($ptr, $count) }
         }
     };
 }
 
 pub(crate) use try_slice;
 
+macro_rules! try_slice_mut {
+    ( $ptr:expr, $count:expr ) => {
+        if $ptr.is_null() {
+            return $crate::panic::NullParameterOrDefault::value();
+        } else {
+            unsafe { slice::from_raw_parts_mut($ptr, $count) }
+        }
+    };
+}
+
+pub(crate) use try_slice_mut;
+
 macro_rules! try_callback {
     ( $var:ident ) => {
         match $var {
@@ -655,11 +723,105 @@ pub extern "C" fn rustls_version() -> rustls_str<'static> {
     rustls_str::from_str_unchecked(RUSTLS_FFI_VERSION)
 }
 
-#[test]
-fn test_rustls_version() {
-    // very rough check that the version number is being interpolated into the
-    // variable
-    assert!(RUSTLS_FFI_VERSION.contains("/0."));
-    let vsn = rustls_version();
-    assert!(vsn.len > 4)
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use std::thread;
+
+    #[test]
+    fn guard_try_pop() {
+        let data = "hello";
+        let data_ptr = data as *const _ as _;
+        let mut guard = userdata_push(data_ptr, None).unwrap();
+        assert_eq!(userdata_get().unwrap(), data_ptr);
+        guard.try_pop().unwrap();
+        assert!(guard.try_pop().is_err())
+    }
+
+    #[test]
+    fn guard_try_drop() {
+        let data = "hello";
+        let data_ptr = data as *const _ as _;
+        let guard = userdata_push(data_ptr, None).unwrap();
+        assert_eq!(userdata_get().unwrap(), data_ptr);
+        guard.try_drop().unwrap();
+        assert!(userdata_get().is_err())
+    }
+
+    #[test]
+    fn guard_drop() {
+        let data = "hello";
+        let data_ptr = data as *const _ as _;
+        {
+            let _guard = userdata_push(data_ptr, None).unwrap();
+            assert_eq!(userdata_get().unwrap(), data_ptr);
+        }
+        assert!(userdata_get().is_err())
+    }
+
+    #[test]
+    fn nested_guards() {
+        let hello = "hello";
+        let hello_ptr = hello as *const _ as _;
+        {
+            let guard = userdata_push(hello_ptr, None).unwrap();
+            assert_eq!(userdata_get().unwrap(), hello_ptr);
+            {
+                let yo = "yo";
+                let yo_ptr = yo as *const _ as _;
+                let guard2 = userdata_push(yo_ptr, None).unwrap();
+                assert_eq!(userdata_get().unwrap(), yo_ptr);
+                guard2.try_drop().unwrap();
+            }
+            assert_eq!(userdata_get().unwrap(), hello_ptr);
+            guard.try_drop().unwrap();
+        }
+        assert!(userdata_get().is_err())
+    }
+
+    #[test]
+    fn out_of_order_drop() {
+        let hello = "hello";
+        let hello_ptr = hello as *const _ as _;
+        let guard = userdata_push(hello_ptr, None).unwrap();
+        assert_eq!(userdata_get().unwrap(), hello_ptr);
+
+        let yo = "yo";
+        let yo_ptr = yo as *const _ as _;
+        let guard2 = userdata_push(yo_ptr, None).unwrap();
+        assert_eq!(userdata_get().unwrap(), yo_ptr);
+
+        assert!(matches!(guard.try_drop(), Err(UserdataError::WrongData)));
+        assert!(matches!(guard2.try_drop(), Err(UserdataError::WrongData)));
+    }
+
+    #[test]
+    fn userdata_multi_threads() {
+        let hello = "hello";
+        let hello_ptr = hello as *const _ as _;
+        let guard = userdata_push(hello_ptr, None).unwrap();
+        assert_eq!(userdata_get().unwrap(), hello_ptr);
+
+        let thread1 = thread::spawn(|| {
+            let yo = "yo";
+            let yo_ptr = yo as *const _ as _;
+            let guard2 = userdata_push(yo_ptr, None).unwrap();
+            assert_eq!(userdata_get().unwrap(), yo_ptr);
+
+            let greetz = "greetz";
+            let greetz_ptr = greetz as *const _ as _;
+
+            let guard3 = userdata_push(greetz_ptr, None).unwrap();
+
+            assert_eq!(userdata_get().unwrap(), greetz_ptr);
+            guard3.try_drop().unwrap();
+
+            assert_eq!(userdata_get().unwrap(), yo_ptr);
+            guard2.try_drop().unwrap();
+        });
+
+        assert_eq!(userdata_get().unwrap(), hello_ptr);
+        guard.try_drop().unwrap();
+        thread1.join().unwrap();
+    }
 }
