@@ -1,48 +1,53 @@
 use std::ffi::c_void;
 use std::fmt::{Debug, Formatter};
-use std::ptr::null;
 use std::slice;
 use std::sync::Arc;
 
 use libc::size_t;
-use rustls::crypto::ring::ALL_CIPHER_SUITES;
+use rustls::crypto::CryptoProvider;
 use rustls::server::danger::ClientCertVerifier;
 use rustls::server::{
     ClientHello, ResolvesServerCert, ServerConfig, ServerConnection, StoresServerSessions,
     WebPkiClientVerifier,
 };
 use rustls::sign::CertifiedKey;
-use rustls::{ProtocolVersion, SignatureScheme, SupportedCipherSuite, WantsVerifier};
+use rustls::{ProtocolVersion, SignatureScheme, SupportedProtocolVersion};
 
-use crate::cipher::{
-    rustls_certified_key, rustls_client_cert_verifier, rustls_supported_ciphersuite,
-};
+use crate::cipher::{rustls_certified_key, rustls_client_cert_verifier};
 use crate::connection::{rustls_connection, Connection};
-use crate::error::rustls_result::{InvalidParameter, NullParameter};
+use crate::crypto_provider::rustls_crypto_provider;
+use crate::error::rustls_result::NullParameter;
 use crate::error::{map_error, rustls_result};
 use crate::rslice::{rustls_slice_bytes, rustls_slice_slice_bytes, rustls_slice_u16, rustls_str};
 use crate::session::{
     rustls_session_store_get_callback, rustls_session_store_put_callback, SessionStoreBroker,
-    SessionStoreGetCallback, SessionStorePutCallback,
 };
 use crate::{
-    ffi_panic_boundary, free_arc, free_box, set_boxed_mut_ptr, to_arc_const_ptr, to_boxed_mut_ptr,
-    try_box_from_ptr, try_clone_arc, try_mut_from_ptr, try_ref_from_ptr, try_slice, userdata_get,
-    Castable, OwnershipArc, OwnershipBox, OwnershipRef,
+    arc_castable, box_castable, crypto_provider, ffi_panic_boundary, free_arc, free_box,
+    set_arc_mut_ptr, set_boxed_mut_ptr, to_boxed_mut_ptr, try_box_from_ptr, try_clone_arc,
+    try_mut_from_ptr, try_mut_from_ptr_ptr, try_ref_from_ptr, try_ref_from_ptr_ptr, try_slice,
+    userdata_get, Castable, OwnershipRef,
 };
 
-/// A server config being constructed. A builder can be modified by,
-/// e.g. rustls_server_config_builder_load_native_roots. Once you're
-/// done configuring settings, call rustls_server_config_builder_build
-/// to turn it into a *const rustls_server_config. This object is not safe
-/// for concurrent mutation.
-/// <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
-pub struct rustls_server_config_builder {
-    _private: [u8; 0],
+box_castable! {
+    /// A server config being constructed.
+    ///
+    /// A builder can be modified by,
+    /// e.g. rustls_server_config_builder_load_native_roots. Once you're
+    /// done configuring settings, call rustls_server_config_builder_build
+    /// to turn it into a *const rustls_server_config.
+    ///
+    /// Alternatively, if an error occurs or, you don't wish to build a config,
+    /// call `rustls_server_config_builder_free` to free the builder directly.
+    ///
+    /// This object is not safe for concurrent mutation.
+    /// <https://docs.rs/rustls/latest/rustls/struct.ConfigBuilder.html>
+    pub struct rustls_server_config_builder(ServerConfigBuilder);
 }
 
 pub(crate) struct ServerConfigBuilder {
-    base: rustls::ConfigBuilder<ServerConfig, WantsVerifier>,
+    provider: Option<Arc<CryptoProvider>>,
+    versions: Vec<&'static SupportedProtocolVersion>,
     verifier: Arc<dyn ClientCertVerifier>,
     cert_resolver: Option<Arc<dyn ResolvesServerCert>>,
     session_storage: Option<Arc<dyn StoresServerSessions + Send + Sync>>,
@@ -50,39 +55,31 @@ pub(crate) struct ServerConfigBuilder {
     ignore_client_order: Option<bool>,
 }
 
-impl Castable for rustls_server_config_builder {
-    type Ownership = OwnershipBox;
-    type RustType = ServerConfigBuilder;
-}
-
-/// A server config that is done being constructed and is now read-only.
-/// Under the hood, this object corresponds to an `Arc<ServerConfig>`.
-/// <https://docs.rs/rustls/latest/rustls/struct.ServerConfig.html>
-pub struct rustls_server_config {
-    _private: [u8; 0],
-}
-
-impl Castable for rustls_server_config {
-    type Ownership = OwnershipArc;
-    type RustType = ServerConfig;
+arc_castable! {
+    /// A server config that is done being constructed and is now read-only.
+    ///
+    /// Under the hood, this object corresponds to an `Arc<ServerConfig>`.
+    /// <https://docs.rs/rustls/latest/rustls/struct.ServerConfig.html>
+    pub struct rustls_server_config(ServerConfig);
 }
 
 impl rustls_server_config_builder {
-    /// Create a rustls_server_config_builder. Caller owns the memory and must
-    /// eventually call rustls_server_config_builder_build, then free the
-    /// resulting rustls_server_config. This uses rustls safe default values
-    /// for the cipher suites, key exchange groups and protocol versions.
+    /// Create a rustls_server_config_builder using the process default crypto provider.
+    ///
+    /// Caller owns the memory and must eventually call rustls_server_config_builder_build,
+    /// then free the resulting rustls_server_config.
+    ///
+    /// Alternatively, if an error occurs or, you don't wish to build a config, call
+    /// `rustls_server_config_builder_free` to free the builder directly.
+    ///
+    /// This uses the process default provider's values for the cipher suites and key exchange
+    /// groups, as well as safe defaults for protocol versions.
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_new() -> *mut rustls_server_config_builder {
         ffi_panic_boundary! {
-            // Unwrap safety: *ring* default provider always has ciphersuites compatible with the
-            // default protocol versions.
-            let base =
-                ServerConfig::builder_with_provider(rustls::crypto::ring::default_provider().into())
-                    .with_safe_default_protocol_versions()
-                    .unwrap();
             let builder = ServerConfigBuilder {
-                base,
+                provider: crypto_provider::get_default_or_install_from_crate_features(),
+                versions: rustls::DEFAULT_VERSIONS.to_vec(),
                 verifier: WebPkiClientVerifier::no_client_auth(),
                 cert_resolver: None,
                 session_storage: None,
@@ -93,44 +90,36 @@ impl rustls_server_config_builder {
         }
     }
 
-    /// Create a rustls_server_config_builder. Caller owns the memory and must
-    /// eventually call rustls_server_config_builder_build, then free the
-    /// resulting rustls_server_config. Specify cipher suites in preference
-    /// order; the `cipher_suites` parameter must point to an array containing
-    /// `len` pointers to `rustls_supported_ciphersuite` previously obtained
-    /// from `rustls_all_ciphersuites_get_entry()`. Set the TLS protocol
-    /// versions to use when negotiating a TLS session.
+    /// Create a rustls_server_config_builder using the specified crypto provider.
     ///
-    /// `tls_version` is the version of the protocol, as defined in rfc8446,
+    /// Caller owns the memory and must eventually call rustls_server_config_builder_build,
+    /// then free the resulting rustls_server_config.
+    ///
+    /// Alternatively, if an error occurs or, you don't wish to build a config, call
+    /// `rustls_server_config_builder_free` to free the builder directly.
+    ///
+    /// `tls_versions` set the TLS protocol versions to use when negotiating a TLS session.
+    ///
+    /// `tls_versions` is the version of the protocol, as defined in rfc8446,
     /// ch. 4.2.1 and end of ch. 5.1. Some values are defined in
     /// `rustls_tls_version` for convenience.
     ///
-    /// `versions` will only be used during the call and the application retains
-    /// ownership. `len` is the number of consecutive `uint16_t` pointed to by `versions`.
+    /// `tls_versions` will only be used during the call and the application retains
+    /// ownership. `tls_versions_len` is the number of consecutive `uint16_t` pointed
+    /// to by `tls_versions`.
+    ///
+    /// Ciphersuites are configured separately via the crypto provider. See
+    /// `rustls_crypto_provider_builder_set_cipher_suites` for more information.
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_new_custom(
-        cipher_suites: *const *const rustls_supported_ciphersuite,
-        cipher_suites_len: size_t,
+        provider: *const rustls_crypto_provider,
         tls_versions: *const u16,
         tls_versions_len: size_t,
         builder_out: *mut *mut rustls_server_config_builder,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            if builder_out.is_null() {
-                return NullParameter;
-            }
-            let cipher_suites: &[*const rustls_supported_ciphersuite] =
-                try_slice!(cipher_suites, cipher_suites_len);
-            let mut cs_vec: Vec<SupportedCipherSuite> = Vec::new();
-            for &cs in cipher_suites.iter() {
-                let cs = try_ref_from_ptr!(cs);
-                match ALL_CIPHER_SUITES.iter().find(|&acs| cs.eq(acs)) {
-                    Some(scs) => cs_vec.push(*scs),
-                    None => return InvalidParameter,
-                }
-            }
-
-            let tls_versions: &[u16] = try_slice!(tls_versions, tls_versions_len);
+            let provider = try_clone_arc!(provider);
+            let tls_versions = try_slice!(tls_versions, tls_versions_len);
             let mut versions = vec![];
             for version_number in tls_versions {
                 let proto = ProtocolVersion::from(*version_number);
@@ -140,20 +129,11 @@ impl rustls_server_config_builder {
                     versions.push(&rustls::version::TLS13);
                 }
             }
-
-            let provider = rustls::crypto::CryptoProvider {
-                cipher_suites: cs_vec,
-                ..rustls::crypto::ring::default_provider()
-            };
-            let result = rustls::ServerConfig::builder_with_provider(provider.into())
-                .with_protocol_versions(&versions);
-            let base = match result {
-                Ok(new) => new,
-                Err(_) => return rustls_result::InvalidParameter,
-            };
+            let builder_out = try_mut_from_ptr_ptr!(builder_out);
 
             let builder = ServerConfigBuilder {
-                base,
+                provider: Some(provider),
+                versions,
                 verifier: WebPkiClientVerifier::no_client_auth(),
                 cert_resolver: None,
                 session_storage: None,
@@ -166,22 +146,26 @@ impl rustls_server_config_builder {
     }
 
     /// Create a rustls_server_config_builder for TLS sessions that may verify client
-    /// certificates. This increases the refcount of `verifier` and doesn't take ownership.
+    /// certificates.
+    ///
+    /// This increases the refcount of `verifier` and doesn't take ownership.
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_set_client_verifier(
         builder: *mut rustls_server_config_builder,
         verifier: *const rustls_client_cert_verifier,
     ) {
         ffi_panic_boundary! {
-            let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
+            let builder = try_mut_from_ptr!(builder);
             let verifier = try_ref_from_ptr!(verifier);
             builder.verifier = verifier.clone();
         }
     }
 
     /// "Free" a server_config_builder without building it into a rustls_server_config.
+    ///
     /// Normally builders are built into rustls_server_configs via `rustls_server_config_builder_build`
     /// and may not be free'd or otherwise used afterwards.
+    ///
     /// Use free only when the building of a config has to be aborted before a config
     /// was created.
     #[no_mangle]
@@ -201,15 +185,16 @@ impl rustls_server_config_builder {
         ignore: bool,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let config: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
+            let config = try_mut_from_ptr!(builder);
             config.ignore_client_order = Some(ignore);
             rustls_result::Ok
         }
     }
 
-    /// Set the ALPN protocol list to the given protocols. `protocols` must point
-    /// to a buffer of `rustls_slice_bytes` (built by the caller) with `len`
-    /// elements. Each element of the buffer must point to a slice of bytes that
+    /// Set the ALPN protocol list to the given protocols.
+    ///
+    /// `protocols` must point to a buffer of `rustls_slice_bytes` (built by the caller)
+    /// with `len` elements. Each element of the buffer must point to a slice of bytes that
     /// contains a single ALPN protocol from
     /// <https://www.iana.org/assignments/tls-extensiontype-values/tls-extensiontype-values.xhtml#alpn-protocol-ids>.
     ///
@@ -224,12 +209,12 @@ impl rustls_server_config_builder {
         len: size_t,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let config: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
-            let protocols: &[rustls_slice_bytes] = try_slice!(protocols, len);
+            let config = try_mut_from_ptr!(builder);
+            let protocols = try_slice!(protocols, len);
 
-            let mut vv: Vec<Vec<u8>> = Vec::new();
+            let mut vv = Vec::new();
             for p in protocols {
-                let v: &[u8] = try_slice!(p.data, p.len);
+                let v = try_slice!(p.data, p.len);
                 vv.push(v.to_vec());
             }
             config.alpn_protocols = vv;
@@ -239,8 +224,10 @@ impl rustls_server_config_builder {
 
     /// Provide the configuration a list of certificates where the connection
     /// will select the first one that is compatible with the client's signature
-    /// verification capabilities. Servers that want to support both ECDSA and
-    /// RSA certificates will want the ECSDA to go first in the list.
+    /// verification capabilities.
+    ///
+    /// Servers that want to support both ECDSA and RSA certificates will want
+    /// the ECSDA to go first in the list.
     ///
     /// The built configuration will keep a reference to all certified keys
     /// provided. The client may `rustls_certified_key_free()` afterwards
@@ -256,12 +243,11 @@ impl rustls_server_config_builder {
         certified_keys_len: size_t,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
-            let keys_ptrs: &[*const rustls_certified_key] =
-                try_slice!(certified_keys, certified_keys_len);
-            let mut keys: Vec<Arc<CertifiedKey>> = Vec::new();
+            let builder = try_mut_from_ptr!(builder);
+            let keys_ptrs = try_slice!(certified_keys, certified_keys_len);
+            let mut keys = Vec::new();
             for &key_ptr in keys_ptrs {
-                let certified_key: Arc<CertifiedKey> = try_clone_arc!(key_ptr);
+                let certified_key = try_clone_arc!(key_ptr);
                 keys.push(certified_key);
             }
             builder.cert_resolver = Some(Arc::new(ResolvesServerCertFromChoices::new(&keys)));
@@ -270,18 +256,38 @@ impl rustls_server_config_builder {
     }
 
     /// Turn a *rustls_server_config_builder (mutable) into a const *rustls_server_config
-    /// (read-only).
+    /// (read-only). The constructed `rustls_server_config` will be written to the `config_out`
+    /// pointer when this function returns `rustls_result::Ok`.
+    ///
+    /// This function may return an error if no process default crypto provider has been set
+    /// and the builder was constructed using `rustls_server_config_builder_new`, or if no
+    /// certificate resolver was set.
     #[no_mangle]
     pub extern "C" fn rustls_server_config_builder_build(
         builder: *mut rustls_server_config_builder,
-    ) -> *const rustls_server_config {
+        config_out: *mut *const rustls_server_config,
+    ) -> rustls_result {
         ffi_panic_boundary! {
             let builder = try_box_from_ptr!(builder);
-            let base = builder.base.with_client_cert_verifier(builder.verifier);
+            let config_out = try_ref_from_ptr_ptr!(config_out);
+
+            let provider = match builder.provider {
+                Some(provider) => provider,
+                None => return rustls_result::NoDefaultCryptoProvider,
+            };
+
+            let base = match ServerConfig::builder_with_provider(provider)
+                .with_protocol_versions(&builder.versions)
+            {
+                Ok(base) => base,
+                Err(err) => return map_error(err),
+            }
+            .with_client_cert_verifier(builder.verifier);
+
             let mut config = if let Some(r) = builder.cert_resolver {
                 base.with_cert_resolver(r)
             } else {
-                return null();
+                return rustls_result::General;
             };
             if let Some(ss) = builder.session_storage {
                 config.session_storage = ss;
@@ -290,14 +296,18 @@ impl rustls_server_config_builder {
             if let Some(ignore_client_order) = builder.ignore_client_order {
                 config.ignore_client_order = ignore_client_order;
             }
-            to_arc_const_ptr(config)
+
+            set_arc_mut_ptr(config_out, config);
+            rustls_result::Ok
         }
     }
 }
 
 impl rustls_server_config {
     /// "Free" a rustls_server_config previously returned from
-    /// rustls_server_config_builder_build. Since rustls_server_config is actually an
+    /// rustls_server_config_builder_build.
+    ///
+    /// Since rustls_server_config is actually an
     /// atomically reference-counted pointer, extant server connections may still
     /// hold an internal reference to the Rust object. However, C code must
     /// consider this pointer unusable after "free"ing it.
@@ -309,12 +319,17 @@ impl rustls_server_config {
         }
     }
 
-    /// Create a new rustls_connection containing a server connection, and return it
-    /// in the output parameter `out`. If this returns an error code, the memory
-    /// pointed to by `conn_out` remains unchanged. If this returns a non-error,
-    /// the memory pointed to by `conn_out` is modified to point
-    /// at a valid rustls_connection. The caller now owns the rustls_connection
-    /// and must call `rustls_connection_free` when done with it.
+    /// Create a new rustls_connection containing a server connection, and return it.
+    ///
+    /// It is returned in the output parameter `conn_out`.
+    ///
+    /// If this returns an error code, the memory pointed to by `conn_out` remains unchanged.
+    ///
+    /// If this returns a non-error, the memory pointed to by `conn_out` is modified to point
+    /// at a valid rustls_connection
+    ///
+    /// The caller now owns the rustls_connection and must call `rustls_connection_free` when
+    /// done with it.
     #[no_mangle]
     pub extern "C" fn rustls_server_connection_new(
         config: *const rustls_server_config,
@@ -324,7 +339,8 @@ impl rustls_server_config {
             if conn_out.is_null() {
                 return NullParameter;
             }
-            let config: Arc<ServerConfig> = try_clone_arc!(config);
+            let config = try_clone_arc!(config);
+            let conn_out = try_mut_from_ptr_ptr!(conn_out);
 
             let server_connection = match ServerConnection::new(config) {
                 Ok(sc) => sc,
@@ -340,10 +356,14 @@ impl rustls_server_config {
     }
 }
 
-/// Copy the server name from the server name indication (SNI) extension to `buf` which can
-/// hold up  to `count` bytes, and the length of that server name in `out_n`. The string is
-/// stored in UTF-8 with no terminating NUL byte.
+/// Copy the server name from the server name indication (SNI) extension to `buf`.
+///
+/// `buf` can hold up  to `count` bytes, and the length of that server name in `out_n`.
+///
+/// The string is stored in UTF-8 with no terminating NUL byte.
+///
 /// Returns RUSTLS_RESULT_INSUFFICIENT_SIZE if the SNI hostname is longer than `count`.
+///
 /// Returns Ok with *out_n == 0 if there is no SNI hostname available on this connection
 /// because it hasn't been processed yet, or because the client did not send SNI.
 /// <https://docs.rs/rustls/latest/rustls/server/struct.ServerConnection.html#method.server_name>
@@ -355,7 +375,7 @@ pub extern "C" fn rustls_server_connection_get_server_name(
     out_n: *mut size_t,
 ) -> rustls_result {
     ffi_panic_boundary! {
-        let conn: &Connection = try_ref_from_ptr!(conn);
+        let conn = try_ref_from_ptr!(conn);
         if buf.is_null() {
             return NullParameter;
         }
@@ -375,7 +395,7 @@ pub extern "C" fn rustls_server_connection_get_server_name(
                 return rustls_result::Ok;
             }
         };
-        let len: usize = sni_hostname.len();
+        let len = sni_hostname.len();
         if len > count {
             unsafe { *out_n = 0 }
             return rustls_result::InsufficientSize;
@@ -420,12 +440,15 @@ impl ResolvesServerCert for ResolvesServerCertFromChoices {
 }
 
 /// The TLS Client Hello information provided to a ClientHelloCallback function.
+///
 /// `server_name` is the value of the ServerNameIndication extension provided
 /// by the client. If the client did not send an SNI, the length of this
-/// `rustls_string` will be 0. The signature_schemes field carries the values
-/// supplied by the client or, if the client did not send this TLS extension,
-/// the default schemes in the rustls library. See:
+/// `rustls_string` will be 0.
+///
+/// `signature_schemes` carries the values supplied by the client or, if the
+/// client did not send this TLS extension, the default schemes in the rustls library. See:
 /// <https://docs.rs/rustls/latest/rustls/internal/msgs/enums/enum.SignatureScheme.html>.
+///
 /// `alpn` carries the list of ALPN protocol names that the client proposed to
 /// the server. Again, the length of this list will be 0 if none were supplied.
 ///
@@ -451,9 +474,13 @@ impl<'a> Castable for rustls_client_hello<'a> {
 pub type rustls_client_hello_userdata = *mut c_void;
 
 /// Prototype of a callback that can be installed by the application at the
-/// `rustls_server_config`. This callback will be invoked by a `rustls_connection`
-/// once the TLS client hello message has been received.
+/// `rustls_server_config`.
+///
+/// This callback will be invoked by a `rustls_connection` once the TLS client
+/// hello message has been received.
+///
 /// `userdata` will be set based on rustls_connection_set_userdata.
+///
 /// `hello` gives the value of the available client announcements, as interpreted
 /// by rustls. See the definition of `rustls_client_hello` for details.
 ///
@@ -495,7 +522,7 @@ impl ClientHelloResolver {
 impl ResolvesServerCert for ClientHelloResolver {
     fn resolve(&self, client_hello: ClientHello) -> Option<Arc<CertifiedKey>> {
         let server_name = client_hello.server_name().unwrap_or_default();
-        let server_name: rustls_str = match server_name.try_into() {
+        let server_name = match server_name.try_into() {
             Ok(r) => r,
             Err(_) => return None,
         };
@@ -511,7 +538,7 @@ impl ResolvesServerCert for ClientHelloResolver {
         };
 
         let alpn = rustls_slice_slice_bytes { inner: &alpn };
-        let signature_schemes: rustls_slice_u16 = (&*mapped_sigs).into();
+        let signature_schemes = (&*mapped_sigs).into();
         let hello = rustls_client_hello {
             server_name,
             signature_schemes,
@@ -523,8 +550,8 @@ impl ResolvesServerCert for ClientHelloResolver {
             Ok(u) => u,
             Err(_) => return None,
         };
-        let key_ptr: *const rustls_certified_key = unsafe { cb(userdata, &hello) };
-        let certified_key: &CertifiedKey = try_ref_from_ptr!(key_ptr);
+        let key_ptr = unsafe { cb(userdata, &hello) };
+        let certified_key = try_ref_from_ptr!(key_ptr);
         Some(Arc::new(certified_key.clone()))
     }
 }
@@ -533,6 +560,7 @@ impl ResolvesServerCert for ClientHelloResolver {
 /// as the registered callbacks are thread safe. This is
 /// documented as a requirement in the API.
 unsafe impl Sync for ClientHelloResolver {}
+
 unsafe impl Send for ClientHelloResolver {}
 
 impl Debug for ClientHelloResolver {
@@ -562,11 +590,11 @@ impl rustls_server_config_builder {
         callback: rustls_client_hello_callback,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let callback: ClientHelloCallback = match callback {
+            let callback = match callback {
                 Some(cb) => cb,
-                None => return rustls_result::NullParameter,
+                None => return NullParameter,
             };
-            let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
+            let builder = try_mut_from_ptr!(builder);
             builder.cert_resolver = Some(Arc::new(ClientHelloResolver::new(callback)));
             rustls_result::Ok
         }
@@ -579,9 +607,10 @@ fn sigschemes(input: &[u16]) -> Vec<SignatureScheme> {
 }
 
 /// Select a `rustls_certified_key` from the list that matches the cryptographic
-/// parameters of a TLS client hello. Note that this does not do any SNI matching.
-/// The input certificates should already have been filtered to ones matching the
-/// SNI from the client hello.
+/// parameters of a TLS client hello.
+///
+/// Note that this does not do any SNI matching. The input certificates should
+/// already have been filtered to ones matching the SNI from the client hello.
 ///
 /// This is intended for servers that are configured with several keys for the
 /// same domain name(s), for example ECDSA and RSA types. The presented keys are
@@ -601,17 +630,16 @@ pub extern "C" fn rustls_client_hello_select_certified_key(
 ) -> rustls_result {
     ffi_panic_boundary! {
         let hello = try_ref_from_ptr!(hello);
-        let schemes: Vec<SignatureScheme> = sigschemes(try_slice!(
+        let schemes = sigschemes(try_slice!(
             hello.signature_schemes.data,
             hello.signature_schemes.len
         ));
         if out_key.is_null() {
             return NullParameter;
         }
-        let keys_ptrs: &[*const rustls_certified_key] =
-            try_slice!(certified_keys, certified_keys_len);
+        let keys_ptrs = try_slice!(certified_keys, certified_keys_len);
         for &key_ptr in keys_ptrs {
-            let key_ref: &CertifiedKey = try_ref_from_ptr!(key_ptr);
+            let key_ref = try_ref_from_ptr!(key_ptr);
             if key_ref.key.choose_scheme(&schemes).is_some() {
                 unsafe {
                     *out_key = key_ptr;
@@ -638,15 +666,15 @@ impl rustls_server_config_builder {
         put_cb: rustls_session_store_put_callback,
     ) -> rustls_result {
         ffi_panic_boundary! {
-            let get_cb: SessionStoreGetCallback = match get_cb {
+            let get_cb = match get_cb {
                 Some(cb) => cb,
-                None => return rustls_result::NullParameter,
+                None => return NullParameter,
             };
-            let put_cb: SessionStorePutCallback = match put_cb {
+            let put_cb = match put_cb {
                 Some(cb) => cb,
-                None => return rustls_result::NullParameter,
+                None => return NullParameter,
             };
-            let builder: &mut ServerConfigBuilder = try_mut_from_ptr!(builder);
+            let builder = try_mut_from_ptr!(builder);
             builder.session_storage = Some(Arc::new(SessionStoreBroker::new(get_cb, put_cb)));
             rustls_result::Ok
         }
@@ -655,50 +683,27 @@ impl rustls_server_config_builder {
 
 #[cfg(test)]
 mod tests {
+    use std::ptr::null;
     use std::ptr::null_mut;
 
     use super::*;
 
     #[test]
+    #[cfg_attr(miri, ignore)]
     fn test_config_builder() {
-        let builder: *mut rustls_server_config_builder =
-            rustls_server_config_builder::rustls_server_config_builder_new();
+        let builder = rustls_server_config_builder::rustls_server_config_builder_new();
         let h1 = "http/1.1".as_bytes();
         let h2 = "h2".as_bytes();
-        let alpn: Vec<rustls_slice_bytes> = vec![h1.into(), h2.into()];
+        let alpn = [h1.into(), h2.into()];
         rustls_server_config_builder::rustls_server_config_builder_set_alpn_protocols(
             builder,
             alpn.as_ptr(),
             alpn.len(),
         );
-        let config = rustls_server_config_builder::rustls_server_config_builder_build(builder);
-        {
-            let config2 = try_ref_from_ptr!(config);
-            assert_eq!(config2.alpn_protocols, vec![h1, h2]);
-        }
-        rustls_server_config::rustls_server_config_free(config);
-    }
 
-    // Build a server connection and test the getters and initial values.
-    #[test]
-    fn test_server_config_builder_new_empty() {
-        let builder: *mut rustls_server_config_builder =
-            rustls_server_config_builder::rustls_server_config_builder_new();
-        // Building a config with no certificate and key configured results in null.
-        assert_eq!(
-            rustls_server_config_builder::rustls_server_config_builder_build(builder),
-            null()
-        );
-    }
-
-    #[test]
-    #[cfg_attr(miri, ignore)]
-    fn test_server_connection_new() {
-        let builder: *mut rustls_server_config_builder =
-            rustls_server_config_builder::rustls_server_config_builder_new();
         let cert_pem = include_str!("../testdata/localhost/cert.pem").as_bytes();
         let key_pem = include_str!("../testdata/localhost/key.pem").as_bytes();
-        let mut certified_key: *const rustls_certified_key = null();
+        let mut certified_key = null();
         let result = rustls_certified_key::rustls_certified_key_build(
             cert_pem.as_ptr(),
             cert_pem.len(),
@@ -718,27 +723,74 @@ mod tests {
             1,
         );
 
-        let config = rustls_server_config_builder::rustls_server_config_builder_build(builder);
-        assert_ne!(config, null());
+        let mut config = null();
+        let result =
+            rustls_server_config_builder::rustls_server_config_builder_build(builder, &mut config);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!config.is_null());
+        {
+            let config2 = try_ref_from_ptr!(config);
+            assert_eq!(config2.alpn_protocols, vec![h1, h2]);
+        }
+        rustls_server_config::rustls_server_config_free(config);
+    }
 
-        let mut conn: *mut rustls_connection = null_mut();
+    // Build a server connection and test the getters and initial values.
+    #[test]
+    fn test_server_config_builder_new_empty() {
+        let builder = rustls_server_config_builder::rustls_server_config_builder_new();
+        // Building a config with no certificate and key configured results in an error.
+        let mut config = null();
+        let result =
+            rustls_server_config_builder::rustls_server_config_builder_build(builder, &mut config);
+        assert_eq!(result, rustls_result::General);
+        assert!(config.is_null());
+    }
+
+    #[test]
+    #[cfg_attr(miri, ignore)]
+    fn test_server_connection_new() {
+        let builder = rustls_server_config_builder::rustls_server_config_builder_new();
+        let cert_pem = include_str!("../testdata/localhost/cert.pem").as_bytes();
+        let key_pem = include_str!("../testdata/localhost/key.pem").as_bytes();
+        let mut certified_key = null();
+        let result = rustls_certified_key::rustls_certified_key_build(
+            cert_pem.as_ptr(),
+            cert_pem.len(),
+            key_pem.as_ptr(),
+            key_pem.len(),
+            &mut certified_key,
+        );
+        if !matches!(result, rustls_result::Ok) {
+            panic!(
+                "expected RUSTLS_RESULT_OK from rustls_certified_key_build, got {:?}",
+                result
+            );
+        }
+        rustls_server_config_builder::rustls_server_config_builder_set_certified_keys(
+            builder,
+            &certified_key,
+            1,
+        );
+
+        let mut config = null();
+        let result =
+            rustls_server_config_builder::rustls_server_config_builder_build(builder, &mut config);
+        assert_eq!(result, rustls_result::Ok);
+        assert!(!config.is_null());
+
+        let mut conn = null_mut();
         let result = rustls_server_config::rustls_server_connection_new(config, &mut conn);
         if !matches!(result, rustls_result::Ok) {
             panic!("expected RUSTLS_RESULT_OK, got {:?}", result);
         }
-        assert_eq!(rustls_connection::rustls_connection_wants_read(conn), true);
-        assert_eq!(
-            rustls_connection::rustls_connection_wants_write(conn),
-            false
-        );
-        assert_eq!(
-            rustls_connection::rustls_connection_is_handshaking(conn),
-            true
-        );
+        assert!(rustls_connection::rustls_connection_wants_read(conn));
+        assert!(!rustls_connection::rustls_connection_wants_write(conn));
+        assert!(rustls_connection::rustls_connection_is_handshaking(conn));
 
         let some_byte = 42u8;
         let mut alpn_protocol: *const u8 = &some_byte;
-        let mut alpn_protocol_len: usize = 1;
+        let mut alpn_protocol_len = 1;
         rustls_connection::rustls_connection_get_alpn_protocol(
             conn,
             &mut alpn_protocol,
@@ -749,8 +801,10 @@ mod tests {
 
         assert_eq!(
             rustls_connection::rustls_connection_get_negotiated_ciphersuite(conn),
-            null()
+            0
         );
+        let cs_name = rustls_connection::rustls_connection_get_negotiated_ciphersuite_name(conn);
+        assert_eq!(unsafe { cs_name.to_str() }, "");
         assert_eq!(
             rustls_connection::rustls_connection_get_peer_certificate(conn, 0),
             null()
